@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 
 #include        "bstdefs.h"
 
+#include        "bstdialog.h"
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -33,10 +35,7 @@ extern "C" {
 #define BST_APP_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), BST_TYPE_APP, BstAppClass))
 #define BST_IS_APP(object)      (GTK_CHECK_TYPE ((object), BST_TYPE_APP))
 #define BST_IS_APP_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_APP))
-
-
-/* --- object & class member/convenience macros --- */
-#define	BST_APP_PROJECT(app)	(BST_APP (app)->project)
+#define BST_APP_GET_CLASS(app)  (G_TYPE_INSTANCE_GET_CLASS ((app), BST_TYPE_APP, BstAppClass))
 
 
 /* --- typedefs --- */
@@ -47,26 +46,29 @@ typedef struct  _BstAppClass  BstAppClass;
 /* --- structures --- */
 struct _BstApp
 {
-  GtkWindow      window;
+  BstDialog      window;
 
-  BseProject	*project;
+  BswProxy	 project;
 
   GtkWidget	*main_vbox;
   GtkNotebook	*notebook;
 
-  GtkWidget	*quit_dialog;
   BseSuper	*default_super;
+
+  GtkWidget	*rack_dialog;
+  GtkWidget	*rack_editor;
 };
 struct _BstAppClass
 {
-  GtkWindowClass        parent_class;
+  BstDialogClass        parent_class;
+  gboolean		seen_apps;
   GSList               *apps;
 };
 
 
 /* --- prototypes --- */
 GtkType		bst_app_get_type		(void);
-BstApp*		bst_app_new			(BseProject	*project);
+BstApp*		bst_app_new			(BswProxy	 project);
 gboolean	bst_app_can_operate		(BstApp         *app,
 						 BstOps		 op);
 void		bst_app_operate			(BstApp         *app,
@@ -77,7 +79,7 @@ GtkWidget*	bst_app_get_current_shell	(BstApp		*app);
 BseSuper*	bst_app_get_current_super	(BstApp		*app);
 GtkItemFactory*	bst_app_menu_factory		(BstApp		*app);
 void		bst_app_update_can_operate	(BstApp		*app);
-     
+BstApp*		bst_app_find			(BswProxy	 project);
 
 
 
