@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  */
 #include "bstcanvaslink.h"
 
+#include "bstdialog.h"
+#include "bsttexttools.h"
+
 #include <math.h>
 
 #define ARROW_LENGTH    (12.0)
@@ -63,7 +66,6 @@ bst_canvas_link_get_type (void)
       };
       
       canvas_link_type = gtk_type_unique (GNOME_TYPE_CANVAS_GROUP, &canvas_link_info);
-      gtk_type_set_chunk_alloc (canvas_link_type, 16);
     }
   
   return canvas_link_type;
@@ -95,10 +97,10 @@ bst_canvas_link_init (BstCanvasLink *clink)
   clink->tag_start = NULL;
   clink->tag_end = NULL;
   clink->ocsource = NULL;
-  clink->ochannel_id = 0;
+  clink->ochannel = ~0;
   clink->oc_handler = 0;
   clink->icsource = NULL;
-  clink->ichannel_id = 0;
+  clink->ichannel = ~0;
   clink->ic_handler = 0;
   clink->link_view = NULL;
 }
@@ -140,9 +142,9 @@ bst_canvas_link_at (GnomeCanvas *canvas,
 }
 
 BstCanvasSource*
-bst_canvas_link_has_canvas_source_at (BstCanvasLink *clink,
-				      gdouble        world_x,
-				      gdouble        world_y)
+bst_canvas_link_csource_at (BstCanvasLink *clink,
+			    gdouble        world_x,
+			    gdouble        world_y)
 {
   GnomeCanvasItem *tag;
 
@@ -162,48 +164,59 @@ clink_view_update (BstCanvasLink *clink,
 		   gboolean       force_update)
 {
   GtkWidget *frame = (clink->link_view && (force_update || GTK_WIDGET_VISIBLE (clink->link_view))
-		      ? bst_adialog_get_child (clink->link_view)
+		      ? bst_dialog_get_child (BST_DIALOG (clink->link_view))
 		      : NULL);
 
   if (frame)
     {
-      GtkWidget *label = GTK_BIN (frame)->child;
-      BseProject *project;
-      BseSourceIChannelDef *ic_def;
-      BseSourceOChannelDef *oc_def;
+      GtkWidget *text = GTK_BIN (frame)->child;
+      gchar *ic_name, *oc_name, *ic_blurb, *oc_blurb;
       gchar *string, *iname, *oname;
 
       /* figure appropriate window title
        */
-      project = clink->icsource ? bse_item_get_project (BSE_ITEM (clink->icsource->source)) : NULL;
-      string = project ? bse_object_get_name_or_type (BSE_OBJECT (project)) : "BEAST";
-      string = g_strconcat (string, ": Source Link", NULL);
-      gtk_window_set_title (GTK_WINDOW (clink->link_view), string);
+      iname = clink->icsource ? bsw_item_get_name_or_type (clink->icsource->source) : "<???>";
+      oname = clink->ocsource ? bsw_item_get_name_or_type (clink->ocsource->source) : "<???>";
+      string = g_strconcat ("Module Link: ", iname, " <=> ", oname, NULL);
+      bst_dialog_set_title (BST_DIALOG (clink->link_view), string);
       g_free (string);
 
       /* construct actuall information
        */
-      iname = clink->icsource ? bse_object_get_name_or_type (BSE_OBJECT (clink->icsource->source)) : "<???>";
-      oname = clink->ocsource ? bse_object_get_name_or_type (BSE_OBJECT (clink->ocsource->source)) : "<???>";
-      oc_def = clink->ocsource ? BSE_SOURCE_OCHANNEL_DEF (clink->ocsource->source, clink->ochannel_id) : NULL;
-      ic_def = clink->icsource ? BSE_SOURCE_ICHANNEL_DEF (clink->icsource->source, clink->ichannel_id) : NULL;
-      string = g_strdup_printf ("Output channel:\n"
-				"        %s:%d %s\n"
-				"        (%s)\n"
-				"Input channel:\n"
-				"        %s:%d %s\n"
-				"        (%s)\n"
-				"\n"
-				"Number of tracks: %d\n"
-				"History: %d\n",
-				oname, clink->ochannel_id, oc_def ? oc_def->name : "?",
-				oc_def ? oc_def->blurb : "?",
-				iname, clink->ichannel_id, ic_def ? ic_def->name : "?",
-				ic_def ? ic_def->blurb : "?",
-				oc_def ? oc_def->n_tracks : -1,
-				ic_def ? ic_def->history : -1);
-      gtk_label_set_text (GTK_LABEL (label), string);
-      g_free (string);
+      oc_name = clink->ocsource ? bsw_source_ochannel_name (clink->ocsource->source, clink->ochannel) : NULL;
+      oc_blurb = clink->ocsource ? bsw_source_ochannel_blurb (clink->ocsource->source, clink->ochannel) : NULL;
+      ic_name = clink->icsource ? bsw_source_ichannel_name (clink->icsource->source, clink->ichannel) : NULL;
+      ic_blurb = clink->icsource ? bsw_source_ichannel_blurb (clink->icsource->source, clink->ichannel) : NULL;
+      if (!oc_name)
+	oc_name = "?";
+      if (!ic_name)
+	ic_name = "?";
+
+      /* compose new info */
+      bst_scroll_text_clear (text);
+      bst_scroll_text_aprintf (text, "Source Module:\n");
+      bst_scroll_text_push_indent (text, "  ");
+      if (oc_blurb)
+	{
+	  bst_scroll_text_aprintf (text, "%s: %s:\n", oname, oc_name);
+	  bst_scroll_text_push_indent (text, "  ");
+	  bst_scroll_text_aprintf (text, "%s\n", oc_blurb);
+	  bst_scroll_text_pop_indent (text);
+	}
+      else
+	bst_scroll_text_aprintf (text, "%s: %s\n", oname, oc_name);
+      bst_scroll_text_pop_indent (text);
+      bst_scroll_text_aprintf (text, "\nDestination Module:\n");
+      bst_scroll_text_push_indent (text, "  ");
+      if (ic_blurb)
+	{
+	  bst_scroll_text_aprintf (text, "%s: %s:\n", iname, ic_name);
+	  bst_scroll_text_push_indent (text, "  ");
+	  bst_scroll_text_aprintf (text, "%s\n", ic_blurb);
+	  bst_scroll_text_pop_indent (text);
+	}
+      else
+	bst_scroll_text_aprintf (text, "%s: %s\n", iname, ic_name);
     }
 }
 
@@ -213,19 +226,16 @@ bst_canvas_link_popup_view (BstCanvasLink *clink)
   g_return_if_fail (BST_IS_CANVAS_LINK (clink));
   
   if (!clink->link_view)
-    clink->link_view = bst_adialog_new (GTK_OBJECT (clink),
-					&clink->link_view,
-					gtk_widget_new (GTK_TYPE_FRAME,
-							"visible", TRUE,
-							"border_width", 5,
-							"label", "Source link",
-							"child", gtk_widget_new (GTK_TYPE_LABEL,
-										 "visible", TRUE,
-										 "justify", GTK_JUSTIFY_LEFT,
-										 "xpad", 5,
-										 NULL),
-							NULL),
-					0, NULL);
+    clink->link_view = bst_dialog_new (&clink->link_view,
+				       GTK_OBJECT (clink),
+				       0,
+				       NULL,
+				       gtk_widget_new (GTK_TYPE_FRAME,
+						       "visible", TRUE,
+						       "border_width", 5,
+						       "label", "Module link",
+						       "child", bst_scroll_text_create (0, NULL),
+						       NULL));
   clink_view_update (clink, TRUE);
   gtk_widget_showraise (clink->link_view);
 }
@@ -252,7 +262,7 @@ clink_view_check_update (BstCanvasLink *clink)
 void
 bst_canvas_link_set_ocsource (BstCanvasLink   *clink,
 			      BstCanvasSource *ocsource,
-			      guint            ochannel_id)
+			      guint            ochannel)
 {
   g_return_if_fail (BST_IS_CANVAS_LINK (clink));
   if (ocsource)
@@ -260,26 +270,26 @@ bst_canvas_link_set_ocsource (BstCanvasLink   *clink,
   
   if (clink->ocsource)
     {
-      bse_object_remove_notifiers_by_func (BSE_OBJECT (clink->ocsource->source),
-					   clink_view_check_update,
-					   clink);
-      if (!GTK_OBJECT_DESTROYED (clink->ocsource))
+      if (clink->ocsource->source) /* source may be destroyed already */
+	bsw_proxy_disconnect (clink->ocsource->source,
+			      "any_signal", clink_view_check_update, clink,
+			      NULL);
+      if (g_signal_handler_is_connected (clink->ocsource, clink->oc_handler))
 	gtk_signal_disconnect (GTK_OBJECT (clink->ocsource), clink->oc_handler);
       gtk_object_unref (GTK_OBJECT (clink->ocsource));
     }
   clink->ocsource = ocsource;
-  clink->ochannel_id = ochannel_id;
+  clink->ochannel = ochannel;
   if (clink->ocsource)
     {
       gtk_object_ref (GTK_OBJECT (clink->ocsource));
       clink->oc_handler = gtk_signal_connect_object (GTK_OBJECT (clink->ocsource),
-						     "args_changed",
-						     bst_canvas_link_update,
+						     "notify",
+						     G_CALLBACK (bst_canvas_link_update),
 						     GTK_OBJECT (clink));
-      bse_object_add_data_notifier (BSE_OBJECT (clink->ocsource->source),
-				    "name-set",
-				    clink_view_check_update,
-				    clink);
+      bsw_proxy_connect (clink->ocsource->source,
+			 "swapped_signal::notify::uname", clink_view_check_update, clink,
+			 NULL);
       bst_canvas_link_update (clink);
     }
 }
@@ -287,7 +297,7 @@ bst_canvas_link_set_ocsource (BstCanvasLink   *clink,
 void
 bst_canvas_link_set_icsource (BstCanvasLink   *clink,
 			      BstCanvasSource *icsource,
-			      guint            ichannel_id)
+			      guint            ichannel)
 {
   g_return_if_fail (BST_IS_CANVAS_LINK (clink));
   if (icsource)
@@ -295,26 +305,26 @@ bst_canvas_link_set_icsource (BstCanvasLink   *clink,
   
   if (clink->icsource)
     {
-      bse_object_remove_notifiers_by_func (BSE_OBJECT (clink->icsource->source),
-					   clink_view_check_update,
-					   clink);
-      if (!GTK_OBJECT_DESTROYED (clink->icsource))
+      if (clink->icsource->source) /* source may be destroyed already */
+	bsw_proxy_disconnect (clink->icsource->source,
+			      "any_signal", clink_view_check_update, clink,
+			      NULL);
+      if (g_signal_handler_is_connected (clink->icsource, clink->ic_handler))
         gtk_signal_disconnect (GTK_OBJECT (clink->icsource), clink->ic_handler);
       gtk_object_unref (GTK_OBJECT (clink->icsource));
     }
   clink->icsource = icsource;
-  clink->ichannel_id = ichannel_id;
+  clink->ichannel = ichannel;
   if (clink->icsource)
     {
       gtk_object_ref (GTK_OBJECT (clink->icsource));
       clink->ic_handler = gtk_signal_connect_object (GTK_OBJECT (clink->icsource),
-						     "args_changed",
-						     bst_canvas_link_update,
+						     "notify",
+						     G_CALLBACK (bst_canvas_link_update),
 						     GTK_OBJECT (clink));
-      bse_object_add_data_notifier (BSE_OBJECT (clink->icsource->source),
-				    "name-set",
-				    clink_view_check_update,
-				    clink);
+      bsw_proxy_connect (clink->icsource->source,
+			 "swapped_signal::notify::uname", clink_view_check_update, clink,
+			 NULL);
       bst_canvas_link_update (clink);
     }
 }
@@ -325,15 +335,17 @@ bst_canvas_link_update (BstCanvasLink *clink)
   GnomeCanvasItem *item = GNOME_CANVAS_ITEM (clink);
   GnomeCanvasPoints *gpoints;
   gdouble start_x = 0, start_y = 0, end_x = 10, end_y = 10;
-  
+  gboolean is_jchannel = FALSE;
+
   if (clink->ocsource)
     {
-      bst_canvas_source_ochannel_pos (clink->ocsource, clink->ochannel_id, &start_x, &start_y);
+      bst_canvas_source_ochannel_pos (clink->ocsource, clink->ochannel, &start_x, &start_y);
       gnome_canvas_item_w2i (item, &start_x, &start_y);
     }
   if (clink->icsource)
     {
-      bst_canvas_source_ichannel_pos (clink->icsource, clink->ichannel_id, &end_x, &end_y);
+      bst_canvas_source_ichannel_pos (clink->icsource, clink->ichannel, &end_x, &end_y);
+      is_jchannel = bst_canvas_source_is_jchannel (clink->icsource, clink->ichannel);
       gnome_canvas_item_w2i (item, &end_x, &end_y);
     }
   if (clink->ocsource && clink->icsource)
@@ -343,45 +355,49 @@ bst_canvas_link_update (BstCanvasLink *clink)
   
   
   if (!clink->arrow)
-    clink->arrow = gnome_canvas_item_new (GNOME_CANVAS_GROUP (clink),
-					  GNOME_TYPE_CANVAS_POLYGON,
-					  "outline_color_rgba", 0x0000ffff,
-					  "fill_color_rgba", 0xff0000ff,
-					  "signal::destroy", gtk_widget_destroyed, &clink->arrow,
-					  "object_signal::event", bst_canvas_link_child_event, clink,
-					  NULL);
+    clink->arrow = g_object_connect (gnome_canvas_item_new (GNOME_CANVAS_GROUP (clink),
+							    GNOME_TYPE_CANVAS_POLYGON,
+							    "outline_color_rgba", 0x0000ffff,
+							    "fill_color_rgba", 0xff0000ff,
+							    NULL),
+				     "signal::destroy", gtk_widget_destroyed, &clink->arrow,
+				     "swapped_signal::event", bst_canvas_link_child_event, clink,
+				     NULL);
   if (!clink->tag_start)
-    clink->tag_start = gnome_canvas_item_new (GNOME_CANVAS_GROUP (clink),
-					      GNOME_TYPE_CANVAS_ELLIPSE,
-					      "outline_color_rgba", 0x000000ff, // xffff00ff,
-					      "fill_color_rgba", 0xffff00ff,
-					      "signal::destroy", gtk_widget_destroyed, &clink->tag_start,
-					      "object_signal::event", bst_canvas_link_child_event, clink,
-					      NULL);
-  if (!clink->tag_end)
-    clink->tag_end = gnome_canvas_item_new (GNOME_CANVAS_GROUP (clink),
-					    GNOME_TYPE_CANVAS_ELLIPSE,
-					    "outline_color_rgba", 0x000000ff, // 0xff0000ff,
-					    "fill_color_rgba", 0xff0000ff,
-					    "signal::destroy", gtk_widget_destroyed, &clink->tag_end,
-					    "object_signal::event", bst_canvas_link_child_event, clink,
-					    NULL);
-  if (!clink->line)
-    clink->line = gnome_canvas_item_new (GNOME_CANVAS_GROUP (clink),
-					 GNOME_TYPE_CANVAS_LINE,
-					 "fill_color", "black",
-					 "signal::destroy", gtk_widget_destroyed, &clink->line,
-					 "object_signal::args_changed", bst_canvas_link_adjust_arrow, clink,
-					 "object_signal::args_changed", bst_canvas_link_adjust_tags, clink,
+    clink->tag_start = g_object_connect (gnome_canvas_item_new (GNOME_CANVAS_GROUP (clink),
+								GNOME_TYPE_CANVAS_ELLIPSE,
+								"outline_color_rgba", 0x000000ff, // xffff00ff,
+								"fill_color_rgba", 0xffff00ff,
+								NULL),
+					 "signal::destroy", gtk_widget_destroyed, &clink->tag_start,
+					 "swapped_signal::event", bst_canvas_link_child_event, clink,
 					 NULL);
+  if (!clink->tag_end)
+    clink->tag_end = g_object_connect (gnome_canvas_item_new (GNOME_CANVAS_GROUP (clink),
+							      GNOME_TYPE_CANVAS_ELLIPSE,
+							      "outline_color_rgba", 0x000000ff, // 0xff0000ff,
+							      "fill_color_rgba", is_jchannel ? 0x00ff00ff : 0xff0000ff,
+							      NULL),
+				       "signal::destroy", gtk_widget_destroyed, &clink->tag_end,
+				       "swapped_signal::event", bst_canvas_link_child_event, clink,
+				       NULL);
+  if (!clink->line)
+    clink->line = g_object_connect (gnome_canvas_item_new (GNOME_CANVAS_GROUP (clink),
+							   GNOME_TYPE_CANVAS_LINE,
+							   "fill_color", "black",
+							   NULL),
+				    "signal::destroy", gtk_widget_destroyed, &clink->line,
+				    "swapped_signal::notify", bst_canvas_link_adjust_arrow, clink,
+				    "swapped_signal::notify", bst_canvas_link_adjust_tags, clink,
+				    NULL);
   gpoints = gnome_canvas_points_new (2);
   gpoints->coords[0] = start_x;
   gpoints->coords[1] = start_y;
   gpoints->coords[2] = end_x;
   gpoints->coords[3] = end_y;
-  bst_object_set (clink->line,
-		  "points", gpoints,
-		  NULL);
+  g_object_set (clink->line,
+		"points", gpoints,
+		NULL);
   gnome_canvas_points_free (gpoints);
   gnome_canvas_item_raise_to_top (clink->tag_start);
   gnome_canvas_item_raise_to_top (clink->tag_end);
@@ -402,22 +418,22 @@ bst_canvas_link_adjust_tags (BstCanvasLink *clink)
   y1 = points[1] - TAG_DIAMETER;
   x2 = points[0] + TAG_DIAMETER;
   y2 = points[1] + TAG_DIAMETER;
-  bst_object_set (clink->tag_start,
-		  "x1", x1,
-		  "y1", y1,
-		  "x2", x2,
-		  "y2", y2,
-		  NULL);
+  g_object_set (clink->tag_start,
+		"x1", x1,
+		"y1", y1,
+		"x2", x2,
+		"y2", y2,
+		NULL);
   x1 = points[2] - TAG_DIAMETER;
   y1 = points[3] - TAG_DIAMETER;
   x2 = points[2] + TAG_DIAMETER;
   y2 = points[3] + TAG_DIAMETER;
-  bst_object_set (clink->tag_end,
-		  "x1", x1,
-		  "y1", y1,
-		  "x2", x2,
-		  "y2", y2,
-		  NULL);
+  g_object_set (clink->tag_end,
+		"x1", x1,
+		"y1", y1,
+		"x2", x2,
+		"y2", y2,
+		NULL);
   
   gnome_canvas_points_free (gpoints);
 }
@@ -460,7 +476,7 @@ bst_canvas_link_adjust_arrow (BstCanvasLink *clink)
   *(points++) = px;
   *(points++) = py;
   
-  bst_object_set (clink->arrow, "points", gpoints, NULL);
+  g_object_set (clink->arrow, "points", gpoints, NULL);
   gnome_canvas_points_free (gpoints);
 }
 
@@ -514,7 +530,7 @@ bst_canvas_link_event (GnomeCanvasItem *item,
 	  gnome_canvas_item_move (GNOME_CANVAS_ITEM (clink->ocsource),
 				  x - clink->start_move_dx,
 				  y - clink->start_move_dy);
-	  BST_OBJECT_ARGS_CHANGED (clink->ocsource);
+	  GNOME_CANVAS_NOTIFY (clink->ocsource);
 	  handled = TRUE;
 	}
       if (clink->in_move && clink->icsource)
@@ -525,7 +541,7 @@ bst_canvas_link_event (GnomeCanvasItem *item,
 	  gnome_canvas_item_move (GNOME_CANVAS_ITEM (clink->icsource),
 				  x - clink->end_move_dx,
 				  y - clink->end_move_dy);
-	  BST_OBJECT_ARGS_CHANGED (clink->icsource);
+	  GNOME_CANVAS_NOTIFY (clink->icsource);
 	  handled = TRUE;
 	}
       break;
@@ -572,19 +588,19 @@ bst_canvas_link_child_event (GnomeCanvasItem *item,
 	{
 	case 'L':
 	  gnome_canvas_item_lower (item, 1);
-	  BST_OBJECT_ARGS_CHANGED (item);
+	  GNOME_CANVAS_NOTIFY (item);
 	  break;
 	case 'l':
 	  gnome_canvas_item_lower_to_bottom (item);
-	  BST_OBJECT_ARGS_CHANGED (item);
+	  GNOME_CANVAS_NOTIFY (item);
 	  break;
 	case 'R':
 	  gnome_canvas_item_raise (item, 1);
-	  BST_OBJECT_ARGS_CHANGED (item);
+	  GNOME_CANVAS_NOTIFY (item);
 	  break;
 	case 'r':
 	  gnome_canvas_item_raise_to_top (item);
-	  BST_OBJECT_ARGS_CHANGED (item);
+	  GNOME_CANVAS_NOTIFY (item);
 	  break;
 	}
       handled = TRUE;
