@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,12 +19,10 @@
 #define __BST_CANVAS_SOURCE_H__
 
 #include	"bstdefs.h"
-#include	<libgnomeui/gnome-canvas.h>
 
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
 
@@ -34,7 +32,7 @@ extern "C" {
 #define BST_CANVAS_SOURCE_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), BST_TYPE_CANVAS_SOURCE, BstCanvasSourceClass))
 #define BST_IS_CANVAS_SOURCE(object)      (GTK_CHECK_TYPE ((object), BST_TYPE_CANVAS_SOURCE))
 #define BST_IS_CANVAS_SOURCE_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_CANVAS_SOURCE))
-#define BST_CANVAS_SOURCE_GET_CLASS(obj)  ((BstCanvasSourceClass*) (((GtkObject*) (obj))->klass))
+#define BST_CANVAS_SOURCE_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), BST_TYPE_CANVAS_SOURCE, BstCanvasSourceClass))
 
 
 /* --- structures & typedefs --- */
@@ -44,16 +42,19 @@ struct _BstCanvasSource
 {
   GnomeCanvasGroup parent_object;
 
-  BseSource       *source;
+  BswProxy	   source;
 
   GtkWidget	  *source_view;
+  GtkWidget	  *source_info;
 
   GnomeCanvasItem *icon_item;
   GnomeCanvasItem *text;
   GSList	  *channel_items;
+  GSList	  *channel_hints;
 
   guint		   swap_channels : 1;
   guint            in_move : 1;
+  guint		   show_hints : 1;
   gdouble	   move_dx, move_dy;
 };
 struct _BstCanvasSourceClass
@@ -67,18 +68,22 @@ struct _BstCanvasSourceClass
 /* --- prototypes --- */
 GtkType		 bst_canvas_source_get_type	(void);
 GnomeCanvasItem* bst_canvas_source_new		(GnomeCanvasGroup *group,
-						 BseSource	  *source,
+						 BswProxy	   source,
 						 gdouble           world_x,
 						 gdouble           world_y);
 void		 bst_canvas_source_update_links	(BstCanvasSource  *csource);
 void		 bst_canvas_source_ochannel_pos	(BstCanvasSource  *csource,
-						 guint             ochannel_id,
+						 guint             ochannel,
 						 gdouble          *world_x,
 						 gdouble          *world_y);
 void		 bst_canvas_source_ichannel_pos	(BstCanvasSource  *csource,
-						 guint             ichannel_id,
+						 guint             ichannel,
 						 gdouble          *world_x,
 						 gdouble          *world_y);
+gboolean	 bst_canvas_source_is_jchannel	(BstCanvasSource  *csource,
+						 guint             ichannel);
+gboolean	 bst_canvas_source_ichannel_free(BstCanvasSource  *csource,
+						 guint		   ichannel);
 guint		 bst_canvas_source_ichannel_at  (BstCanvasSource  *csource,
 						 gdouble           world_x,
 						 gdouble           world_y);
@@ -89,14 +94,17 @@ BstCanvasSource* bst_canvas_source_at  		(GnomeCanvas      *csource,
 						 gdouble           world_x,
 						 gdouble           world_y);
 void		 bst_canvas_source_popup_view	(BstCanvasSource  *csource);
-void		 bst_canvas_source_toggle_view	(BstCanvasSource  *canvas);
+void		 bst_canvas_source_toggle_view	(BstCanvasSource  *csource);
+void		 bst_canvas_source_popup_info	(BstCanvasSource  *csource);
+void		 bst_canvas_source_toggle_info	(BstCanvasSource  *csource);
+void	   bst_canvas_source_set_channel_hints	(BstCanvasSource  *csource,
+						 gboolean          on_off);
 
      
      
 
 
 #ifdef __cplusplus
-#pragma {
 }
 #endif /* __cplusplus */
 
