@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,14 +19,13 @@
 #define __BST_DEFS_H__
 
 #include	"bstutils.h"
-#include	"gnomeforest.h"
+#include	"gtklistwrapper.h"
 #include	"glewidgets.h"
 #include	"bstzoomedwindow.h"
 #include	"bstfreeradiobutton.h"
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
 
@@ -43,13 +42,16 @@ typedef enum
   BST_OP_PROJECT_SAVE_AS,
   BST_OP_PROJECT_NEW_SONG,
   BST_OP_PROJECT_NEW_SNET,
+  BST_OP_PROJECT_NEW_MIDI_SYNTH,
   BST_OP_PROJECT_CLOSE,
   BST_OP_PROJECT_PLAY,
   BST_OP_PROJECT_STOP,
+  BST_OP_PROJECT_RACK_EDITOR,
 
   /* spawn new dialogs
    */
   BST_OP_DIALOG_PREFERENCES,
+  BST_OP_DIALOG_PROC_BROWSER,
   BST_OP_DIALOG_DEVICE_MONITOR,
 
   /* debugging */
@@ -58,11 +60,14 @@ typedef enum
 
   /* song operations
    */
-  BST_OP_PATTERN_ADD,
-  BST_OP_PATTERN_DELETE,
-  BST_OP_PATTERN_EDITOR,
-  BST_OP_INSTRUMENT_ADD,
-  BST_OP_INSTRUMENT_DELETE,
+  BST_OP_PART_ADD,
+  BST_OP_PART_DELETE,
+  BST_OP_PART_EDITOR,
+  BST_OP_WAVE_LOAD,
+  BST_OP_WAVE_DELETE,
+  BST_OP_WAVE_EDITOR,
+  BST_OP_TRACK_ADD,
+  BST_OP_TRACK_DELETE,
 
   /* super operations
    */
@@ -75,13 +80,14 @@ typedef enum
 
   /* help dialogs
    */
-#define BST_OP_HELP_FIRST	BST_OP_HELP_ABOUT
-  BST_OP_HELP_ABOUT,
+#define BST_OP_HELP_FIRST	BST_OP_HELP_FAQ
   BST_OP_HELP_FAQ,
   BST_OP_HELP_KEYTABLE,
-  BST_OP_HELP_HEART,
-  BST_OP_HELP_NETWORKS,
-#define	BST_OP_HELP_LAST	BST_OP_HELP_NETWORKS
+  BST_OP_HELP_RELEASE_NOTES,
+  BST_OP_HELP_GSL_PLAN,
+  BST_OP_HELP_QUICK_START,
+  BST_OP_HELP_ABOUT,
+#define	BST_OP_HELP_LAST	BST_OP_HELP_ABOUT
 
   BST_OP_LAST
 } BstOps;
@@ -104,6 +110,12 @@ typedef enum
 #define BST_DRAG_BUTTON_MOVE_MASK (GDK_BUTTON2_MASK)
 #define BST_DRAG_BUTTON_CONTEXT   (3) /* delete, clone, linkdup */
 
+
+/* --- GUI padding --- */
+#define	BST_OUTER_PADDING	  (2)
+#define	BST_INNER_PADDING	  (3)
+
+
 /* --- debug stuff --- */
 typedef enum                    /*< skip >*/
 { /* keep in sync with bstmain.c */
@@ -116,29 +128,26 @@ extern BstDebugFlags bst_debug_flags;
 #else  /* !G_ENABLE_DEBUG */
 #  define BST_IF_DEBUG(type)	while (0) /* don't exec */
 #endif /* !G_ENABLE_DEBUG */
-
-
+#define	BST_DVL_EXT		(BSE_DVL_EXT)
+#define	BST_DVL_HINTS		(bst_dvl_hints)
+extern gboolean bst_dvl_hints;
 
 extern void bst_update_can_operate (GtkWidget   *some_widget);
 
-/* it's hackish to have these prototypes in here, but we need
- * 'em somewhere, implementations are in bstmain.c
- */
-extern void bst_object_set         (gpointer     object,
-				    const gchar *first_arg_name,
-				    ...); /* hackery rulez! */
-#define	BST_OBJECT_ARGS_CHANGED(object)	G_STMT_START { \
-    if (!GTK_OBJECT_DESTROYED (object)) \
-      gtk_signal_emit_by_name ((GtkObject*) (object), "args-changed"); \
+#define	GNOME_CANVAS_NOTIFY(object)	G_STMT_START { \
+    if (GTK_IS_OBJECT (object)) \
+      g_signal_emit_by_name (object, "notify::generic-change", NULL); \
 } G_STMT_END
 
+#define bst_nullify_on_destroy(obj, location) \
+  g_signal_connect_swapped ((obj), "destroy", G_CALLBACK (g_nullify_pointer), (location))
 
+#define	BST_TOOLTIPS	(bst_global_tooltips ())
 
 
 
 
 #ifdef __cplusplus
-#pragma {
 }
 #endif /* __cplusplus */
 
