@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,13 +19,13 @@
 
 #include	"bststatusbar.h"
 #include	"bstmenus.h"
+#include	<unistd.h>
 #include	<errno.h>
 
 
 /* --- prototypes --- */
 static void	bst_file_dialog_class_init	(BstFileDialogClass	*class);
 static void	bst_file_dialog_init		(BstFileDialog		*fd);
-static void	bst_file_dialog_destroy		(GtkObject		*object);
 
 
 /* --- variables --- */
@@ -75,55 +75,34 @@ bst_file_dialog_init (BstFileDialog *fd)
   gtk_file_selection_heal (GTK_FILE_SELECTION (fd));
 }
 
-static void
-bst_file_dialog_destroy (GtkObject *object)
-{
-  GTK_OBJECT_CLASS (parent_class)->destroy (object);
-}
-
 static void
 bst_file_dialog_open (BstFileDialog *fd)
 {
   gchar *file_name;
-  BseStorage *storage;
+  BswProxy project;
   BseErrorType error;
-  BstApp *app = NULL;
 
   file_name = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fd)));
 
-  storage = bse_storage_new ();
-  error = bse_storage_input_file (storage, file_name);
+  project = bsw_server_use_new_project (BSW_SERVER, file_name);
+  error = bsw_project_restore_from_file (project, file_name);
 
   if (error)
-    {
-      g_message ("failed to open `%s': %s", /* FIXME */
-		 file_name,
-		 bse_error_blurb (error));
-      bst_status_printf (0, bse_error_blurb (error), "Failed to open `%s'", file_name);
-    }
+    g_message ("failed to load project `%s': %s", /* FIXME */
+	       file_name,
+	       bse_error_blurb (error));
   else
     {
-      BseProject *project = bse_project_new (file_name);
+      BstApp *app;
 
-      bse_storage_set_path_resolver (storage, bse_project_path_resolver, project);
-      
-      error = bse_project_restore (project, storage);
-      if (error)
-	g_message ("failed to load project `%s': %s", /* FIXME */
-		   file_name,
-		   bse_error_blurb (error));
+      bsw_project_ensure_wave_repo (project);
       app = bst_app_new (project);
       bst_status_window_push (app);
-      bse_object_unref (BSE_OBJECT (project));
-      bst_status_printf (error ? 0 : 100,
-			 error ? "Failed" : "Done",
-			 "Loading project `%s'",
-			 file_name);
+      bst_status_eprintf (error,"Loading project `%s'", file_name);
       bst_status_window_pop ();
       gtk_idle_show_widget (GTK_WIDGET (app));
     }
-  
-  bse_storage_destroy (storage);
+  bsw_item_unuse (project);
 
   g_free (file_name);
   gtk_widget_destroy (GTK_WIDGET (fd));
@@ -136,36 +115,37 @@ bst_file_dialog_save (BstFileDialog *fd)
   BstApp *app;
   gchar *file_name;
   GtkWidget *radio;
-  BseErrorType error;
-  
+  BswErrorType error;
+  gboolean self_contained = FALSE;
+
   file_name = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fd)));
   app = gtk_object_get_data (GTK_OBJECT (fd), "app");
 
   radio = gtk_object_get_data (GTK_OBJECT (fd), "radio-1");
-  if (radio && GTK_TOGGLE_BUTTON (radio)->active)
-    ;
+  if (radio)
+    self_contained = !GTK_TOGGLE_BUTTON (radio)->active;
 
   bst_status_window_push (app);
 
  retry_saving:
 
-  error = bse_project_store_bse (app->project, file_name);
+  error = bsw_project_store_bse (app->project, file_name, self_contained);
 
   /* offer retry if file exists
    */
   if (error == BSE_ERROR_FILE_EXISTS)
     {
       GtkWidget *choice;
-      gchar *title = g_strdup_printf ("Saving project `%s'", BSE_OBJECT_NAME (app->project));
+      gchar *title = g_strdup_printf ("Saving project `%s'", bsw_item_get_name (app->project));
       gchar *text = g_strdup_printf ("Failed to save\n`%s'\nto\n`%s':\n%s",
-				     BSE_OBJECT_NAME (app->project),
+				     bsw_item_get_name (app->project),
 				     file_name,
 				     bse_error_blurb (error));
       
       choice = bst_choice_dialog_createv (BST_CHOICE_TITLE (title),
 					  BST_CHOICE_TEXT (text),
-					  BST_CHOICE_D (1, "Overwrite", NONE),
-					  BST_CHOICE (0, "Cancel", NONE),
+					  BST_CHOICE_D (1, BST_STOCK_OVERWRITE, NONE),
+					  BST_CHOICE (0, BST_STOCK_CANCEL, NONE),
 					  BST_CHOICE_END);
       g_free (title);
       g_free (text);
@@ -173,7 +153,7 @@ bst_file_dialog_save (BstFileDialog *fd)
 	{
 	  bst_choice_destroy (choice);
 	  if (unlink (file_name) < 0)
-	    bst_status_printf (0, g_strerror (errno), "Deleting `%s'", file_name);
+	    bst_status_errnoprintf (errno, "Deleting `%s'", file_name);
 	  else
 	    goto retry_saving;
 	}
@@ -182,10 +162,7 @@ bst_file_dialog_save (BstFileDialog *fd)
     }
   else
     {
-      bst_status_printf (error ? 0 : 100,
-			 error ? bse_error_blurb (error) : "Done",
-			 "Saving project `%s'",
-			 file_name);
+      bst_status_eprintf (error, "Saving project `%s'", file_name);
       if (!error)
 	gtk_widget_destroy (GTK_WIDGET (fd));
     }
@@ -200,7 +177,7 @@ bst_file_dialog_new_open (BstApp *app)
   GtkWidget *dialog;
 
   dialog = gtk_widget_new (BST_TYPE_FILE_DIALOG,
-			   "title", "BEAST Open Project",
+			   "title", "Open Project: BEAST",
 			   NULL);
   gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (dialog)->ok_button),
 			     "clicked",
@@ -215,7 +192,7 @@ bst_file_dialog_new_open (BstApp *app)
       gtk_object_set_data (GTK_OBJECT (dialog), "app", app);
       gtk_signal_connect_object_while_alive (GTK_OBJECT (app),
 					     "destroy",
-					     gtk_widget_destroy,
+					     G_CALLBACK (gtk_widget_destroy),
 					     GTK_OBJECT (dialog));
     }
 
@@ -234,7 +211,7 @@ bst_file_dialog_new_save (BstApp *app)
 
   g_return_val_if_fail (BST_IS_APP (app), NULL);
 
-  string = g_strconcat ("BEAST Save Project: ", BSE_OBJECT_NAME (app->project), NULL);
+  string = g_strconcat ("Save Project: ", bsw_item_get_name (app->project), ": BEAST", NULL);
   dialog = gtk_widget_new (BST_TYPE_FILE_DIALOG,
 			   "title", string,
 			   NULL);
@@ -250,7 +227,7 @@ bst_file_dialog_new_save (BstApp *app)
 			 "parent", frame,
 			 NULL);
   radio1 = gtk_widget_new (GTK_TYPE_RADIO_BUTTON,
-			   "label", "Checkbox1",
+			   "label", "Store references to wave files",
 			   "visible", TRUE,
 			   "parent", vbox,
 			   "can_focus", FALSE,
@@ -258,7 +235,7 @@ bst_file_dialog_new_save (BstApp *app)
   gtk_misc_set_alignment (GTK_MISC (GTK_BIN (radio1)->child), 0, .5);
   gtk_object_set_data (GTK_OBJECT (dialog), "radio-1", radio1);
   radio2 = gtk_widget_new (GTK_TYPE_RADIO_BUTTON,
-			   "label", "Checkbox2",
+			   "label", "Include wave files",
 			   "visible", TRUE,
 			   "parent", vbox,
 			   "group", radio1,
@@ -278,7 +255,7 @@ bst_file_dialog_new_save (BstApp *app)
   gtk_object_set_data (GTK_OBJECT (dialog), "app", app);
   gtk_signal_connect_object_while_alive (GTK_OBJECT (app),
 					 "destroy",
-					 gtk_widget_destroy,
+					 G_CALLBACK (gtk_widget_destroy),
 					 GTK_OBJECT (dialog));
 
   return dialog;
