@@ -1,18 +1,18 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1999, 2000 Tim Janik and Red Hat, Inc.
+ * Copyright (C) 1999-2002 Tim Janik and Red Hat, Inc.
  *
- * This library is free software; you can redistribute it and/or
+ * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
+ * Public License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
@@ -31,36 +31,41 @@ enum
   PARAM_DISABLE_ALSA,
   PARAM_TAB_WIDTH,
   PARAM_SAMPLE_SWEEP,
-  PARAM_PE_KEY_FOCUS_UNSELECTS
+  PARAM_PE_KEY_FOCUS_UNSELECTS,
+  PARAM_RC_VERSION
 };
 
 
 /* --- prototypes --- */
 static void	 bst_gconfig_init		(BstGConfig	  *gconf);
 static void	 bst_gconfig_class_init		(BstGConfigClass  *class);
-static void	 bst_gconfig_class_destroy	(BstGConfigClass  *class);
-static void      bst_gconfig_set_param          (BstGConfig	  *gconf,
-						 BseParam         *param,
-						 guint             param_id);
-static void      bst_gconfig_get_param          (BstGConfig	  *gconf,
-						 BseParam         *param,
-						 guint             param_id);
-static void	 bst_gconfig_do_shutdown	(BseObject     	  *object);
-static void	 bst_gconfig_do_destroy		(BseObject     	  *object);
+static void	 bst_gconfig_class_finalize	(BstGConfigClass  *class);
+static void      bst_gconfig_set_property          (BstGConfig	  *gconf,
+						 guint             param_id,
+						 GValue           *value,
+						 GParamSpec       *pspec,
+						 const gchar      *trailer);
+static void      bst_gconfig_get_property          (BstGConfig	  *gconf,
+						 guint             param_id,
+						 GValue           *value,
+						 GParamSpec       *pspec,
+						 const gchar      *trailer);
+static void	 bst_gconfig_do_finalize	(GObject     	  *object);
 static void	 bst_gconfig_do_apply		(BseGConfig	  *bconf);
 static void	 bst_gconfig_do_revert		(BseGConfig	  *bconf);
 static void      bst_globals_copy               (const BstGlobals *globals_src,
 						 BstGlobals       *globals);
-static void      bst_globals_reset              (BstGlobals       *globals);
+static void      bst_globals_unset              (BstGlobals       *globals);
 
 
 
 /* --- variables --- */
-BseType                  bst_type_id_BstGConfig = 0;
-static BseTypeClass     *parent_class = NULL;
+GType                    bst_type_id_BstGConfig = 0;
+static gpointer          parent_class = NULL;
 static BstGlobals        bst_globals_current = { 0, };
 const BstGlobals * const bst_globals = &bst_globals_current;
 static const BstGlobals  bst_globals_defaults = {
+  NULL			/* rc_version */,
   NULL			/* xkb_symbol */,
   FALSE			/* xkb_force_query */,
   TRUE			/* snet_anti_aliased */,
@@ -77,18 +82,18 @@ static const BstGlobals  bst_globals_defaults = {
 void
 bst_globals_init (void)
 {
-  static const BseTypeInfo gconfig_info = {
+  static const GTypeInfo gconfig_info = {
     sizeof (BstGConfigClass),
     
-    (BseBaseInitFunc) NULL,
-    (BseBaseDestroyFunc) NULL,
-    (BseClassInitFunc) bst_gconfig_class_init,
-    (BseClassDestroyFunc) bst_gconfig_class_destroy,
+    (GBaseInitFunc) NULL,
+    (GBaseFinalizeFunc) NULL,
+    (GClassInitFunc) bst_gconfig_class_init,
+    (GClassFinalizeFunc) bst_gconfig_class_finalize,
     NULL /* class_data */,
 
     sizeof (BstGConfig),
     0 /* n_preallocs */,
-    (BseObjectInitFunc) bst_gconfig_init,
+    (GInstanceInitFunc) bst_gconfig_init,
   };
 
   if (bst_type_id_BstGConfig)
@@ -102,7 +107,7 @@ bst_globals_init (void)
 }
 
 static void
-bst_gconfig_class_destroy (BstGConfigClass *class)
+bst_gconfig_class_finalize (BstGConfigClass *class)
 {
 }
 
@@ -113,42 +118,30 @@ bst_gconfig_init (BstGConfig *gconf)
 }
 
 static void
-bst_gconfig_do_shutdown (BseObject *object)
-{
-  BstGConfig *gconf;
-  
-  gconf = BST_GCONFIG (object);
-  
-  /* chain parent class' shutdown handler */
-  BSE_OBJECT_CLASS (parent_class)->shutdown (object);
-}
-
-static void
-bst_gconfig_do_destroy (BseObject *object)
+bst_gconfig_do_finalize (GObject *object)
 {
   BstGConfig *gconf = BST_GCONFIG (object);
 
-  bst_globals_reset (&gconf->globals);
+  bst_globals_unset (&gconf->globals);
   
-  /* chain parent class' destroy handler */
-  BSE_OBJECT_CLASS (parent_class)->destroy (object);
+  /* chain parent class' finalize handler */
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
 bst_gconfig_class_init (BstGConfigClass *class)
 {
-  BseObjectClass *object_class;
-  BseGConfigClass *bconfig_class;
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
+  BseGConfigClass *bconfig_class = BSE_GCONFIG_CLASS (class);
   BstGlobals globals_defaults = { 0, };
   
-  parent_class = bse_type_class_peek (BSE_TYPE_GCONFIG);
-  object_class = BSE_OBJECT_CLASS (class);
-  bconfig_class = BSE_GCONFIG_CLASS (class);
+  parent_class = g_type_class_peek (BSE_TYPE_GCONFIG);
   
-  object_class->set_param = (BseObjectSetParamFunc) bst_gconfig_set_param;
-  object_class->get_param = (BseObjectGetParamFunc) bst_gconfig_get_param;
-  object_class->shutdown = bst_gconfig_do_shutdown;
-  object_class->destroy = bst_gconfig_do_destroy;
+  G_OBJECT_CLASS (class)->finalize = bst_gconfig_do_finalize;
+
+  gobject_class->set_property = (GObjectSetPropertyFunc) bst_gconfig_set_property;
+  gobject_class->get_property = (GObjectGetPropertyFunc) bst_gconfig_get_property;
   
   bconfig_class->apply = bst_gconfig_do_apply;
   bconfig_class->revert = bst_gconfig_do_revert;
@@ -159,6 +152,11 @@ bst_gconfig_class_init (BstGConfigClass *class)
 			      bse_param_spec_bool ("xkb_force_query", "Always query X server on startup", NULL,
 						   globals_defaults.xkb_force_query,
 						   BSE_PARAM_DEFAULT));
+  bse_object_class_add_param (object_class, "Internal",
+			      PARAM_RC_VERSION,
+			      bse_param_spec_string ("rc_version", "RC Version", NULL,
+						     globals_defaults.rc_version,
+						     BSE_PARAM_STORAGE));
   bse_object_class_add_param (object_class, "Keyboard Layout",
 			      PARAM_XKB_SYMBOL,
 			      bse_param_spec_string ("xkb_symbol", "Keyboard Layout", NULL,
@@ -200,94 +198,104 @@ bst_gconfig_class_init (BstGConfigClass *class)
 						   "tabulators that show the song, network or sample names. "
 						   "Setting it to a fixed width avoids window resizing when "
 						   "samples are added or removed.",
-						   0, 1024, 5, globals_defaults.tab_width,
+						   0, 1024, globals_defaults.tab_width, 5,
 						   BSE_PARAM_DEFAULT));
   bse_object_class_add_param (object_class, "Debugging",
 			      PARAM_DISABLE_ALSA,
 			      bse_param_spec_bool ("disable_alsa", "Disable support for ALSA PCM driver", NULL,
 						   globals_defaults.disable_alsa,
 						   BSE_PARAM_DEFAULT));
-  bst_globals_reset (&globals_defaults);
+  bst_globals_unset (&globals_defaults);
 }
 
 static void
-bst_gconfig_set_param (BstGConfig *gconf,
-		       BseParam   *param,
-		       guint       param_id)
+bst_gconfig_set_property (BstGConfig  *gconf,
+			  guint        param_id,
+			  GValue      *value,
+			  GParamSpec  *pspec,
+			  const gchar *trailer)
 {
   switch (param_id)
     {
     case PARAM_SNET_ANTI_ALIASED:
-      gconf->globals.snet_anti_aliased = param->value.v_bool;
+      gconf->globals.snet_anti_aliased = g_value_get_boolean (value);
       break;
     case PARAM_SNET_EDIT_FALLBACK:
-      gconf->globals.snet_edit_fallback = param->value.v_bool;
+      gconf->globals.snet_edit_fallback = g_value_get_boolean (value);
       break;
     case PARAM_SNET_SWAP_IO_CHANNELS:
-      gconf->globals.snet_swap_io_channels = param->value.v_bool;
+      gconf->globals.snet_swap_io_channels = g_value_get_boolean (value);
       break;
     case PARAM_XKB_FORCE_QUERY:
-      gconf->globals.xkb_force_query = param->value.v_bool;
+      gconf->globals.xkb_force_query = g_value_get_boolean (value);
+      break;
+    case PARAM_RC_VERSION:
+      g_free (gconf->globals.rc_version);
+      gconf->globals.rc_version = g_value_dup_string (value);
       break;
     case PARAM_XKB_SYMBOL:
       g_free (gconf->globals.xkb_symbol);
-      gconf->globals.xkb_symbol = bse_strdup_stripped (param->value.v_string);
+      gconf->globals.xkb_symbol = bse_strdup_stripped (g_value_get_string (value));
       break;
     case PARAM_DISABLE_ALSA:
-      gconf->globals.disable_alsa = param->value.v_bool;
+      gconf->globals.disable_alsa = g_value_get_boolean (value);
       break;
     case PARAM_TAB_WIDTH:
-      gconf->globals.tab_width = param->value.v_uint;
+      gconf->globals.tab_width = g_value_get_uint (value);
       break;
     case PARAM_SAMPLE_SWEEP:
-      gconf->globals.sample_sweep = param->value.v_bool;
+      gconf->globals.sample_sweep = g_value_get_boolean (value);
       break;
     case PARAM_PE_KEY_FOCUS_UNSELECTS:
-      gconf->globals.pe_key_focus_unselects = param->value.v_bool;
+      gconf->globals.pe_key_focus_unselects = g_value_get_boolean (value);
       break;
     default:
-      BSE_UNHANDLED_PARAM_ID (gconf, param, param_id);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gconf, param_id, pspec);
       break;
     }
 }
 
 static void
-bst_gconfig_get_param (BstGConfig *gconf,
-		       BseParam   *param,
-		       guint       param_id)
+bst_gconfig_get_property (BstGConfig  *gconf,
+                       guint        param_id,
+		       GValue      *value,
+		       GParamSpec  *pspec,
+		       const gchar *trailer)
 {
   switch (param_id)
     {
     case PARAM_SNET_ANTI_ALIASED:
-      param->value.v_bool = gconf->globals.snet_anti_aliased;
+      g_value_set_boolean (value, gconf->globals.snet_anti_aliased);
       break;
     case PARAM_SNET_EDIT_FALLBACK:
-      param->value.v_bool = gconf->globals.snet_edit_fallback;
+      g_value_set_boolean (value, gconf->globals.snet_edit_fallback);
       break;
     case PARAM_SNET_SWAP_IO_CHANNELS:
-      param->value.v_bool = gconf->globals.snet_swap_io_channels;
+      g_value_set_boolean (value, gconf->globals.snet_swap_io_channels);
       break;
     case PARAM_XKB_FORCE_QUERY:
-      param->value.v_bool = gconf->globals.xkb_force_query;
+      g_value_set_boolean (value, gconf->globals.xkb_force_query);
+      break;
+    case PARAM_RC_VERSION:
+      g_value_set_string (value, gconf->globals.rc_version);
       break;
     case PARAM_XKB_SYMBOL:
-      g_free (param->value.v_string);
-      param->value.v_string = g_strdup (gconf->globals.xkb_symbol);
+      g_value_set_string (value, gconf->globals.xkb_symbol);
       break;
     case PARAM_DISABLE_ALSA:
-      param->value.v_bool = gconf->globals.disable_alsa;
+      g_value_set_boolean (value, gconf->globals.disable_alsa);
       break;
     case PARAM_TAB_WIDTH:
-      param->value.v_uint = gconf->globals.tab_width;
+      g_value_set_uint (value, gconf->globals.tab_width);
       break;
     case PARAM_SAMPLE_SWEEP:
-      param->value.v_bool = gconf->globals.sample_sweep;
+      g_value_set_boolean (value, gconf->globals.sample_sweep);
       break;
     case PARAM_PE_KEY_FOCUS_UNSELECTS:
-      param->value.v_bool = gconf->globals.pe_key_focus_unselects;
+      g_value_set_boolean (value, gconf->globals.pe_key_focus_unselects);
       break;
     default:
-      BSE_UNHANDLED_PARAM_ID (gconf, param, param_id);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gconf, param_id, pspec);
       break;
     }
 }
@@ -308,7 +316,7 @@ bst_gconfig_do_revert (BseGConfig *bconf)
 {
   BstGConfig *gconf = BST_GCONFIG (bconf);
 
-  bst_globals_reset (&gconf->globals);
+  bst_globals_unset (&gconf->globals);
   bst_globals_copy (bst_globals, &gconf->globals);
 
   /* chain parent class' handler */
@@ -325,23 +333,34 @@ bst_globals_copy (const BstGlobals *globals_src,
     {
       g_return_if_fail (bse_globals_locked () == FALSE);
 
-      bst_globals_reset (&bst_globals_current);
+      bst_globals_unset (&bst_globals_current);
       globals = &bst_globals_current;
     }
 
   *globals = *globals_src;
+  globals->rc_version = g_strdup (globals_src->rc_version);
   globals->xkb_symbol = g_strdup (globals_src->xkb_symbol);
 }
 
 void
-bst_globals_reset (BstGlobals *globals)
+bst_globals_unset (BstGlobals *globals)
 {
   g_return_if_fail (globals != NULL);
 
+  g_free (globals->rc_version);
   g_free (globals->xkb_symbol);
   memset (globals, 0, sizeof (*globals));
 }
 
+void
+bst_globals_set_rc_version (const gchar *rc_version)
+{
+  g_return_if_fail (bse_globals_locked () == FALSE);
+
+  g_free (bst_globals_current.rc_version);
+  bst_globals_current.rc_version = g_strdup (rc_version);
+}
+
 void
 bst_globals_set_xkb_symbol (const gchar *xkb_symbol)
 {
