@@ -1,18 +1,18 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1999, 2000 Tim Janik and Red Hat, Inc.
+ * Copyright (C) 1999-2002 Tim Janik and Red Hat, Inc.
  *
- * This library is free software; you can redistribute it and/or
+ * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
+ * Public License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
@@ -29,6 +29,7 @@ extern "C" {
 
 /* --- BstGlobals - configurable defaults --- */
 #define BST_XKB_FORCE_QUERY		(bst_globals->xkb_force_query)
+#define	BST_RC_VERSION			(bst_globals->rc_version)
 #define BST_XKB_SYMBOL			(bst_globals->xkb_symbol)
 #define BST_DISABLE_ALSA		(bst_globals->disable_alsa)
 #define BST_TAB_WIDTH			(bst_globals->tab_width)
@@ -43,6 +44,7 @@ extern "C" {
 typedef struct _BstGlobals BstGlobals;
 struct _BstGlobals
 {
+  gchar *rc_version;
   gchar *xkb_symbol;
   guint  xkb_force_query : 1;
   guint  snet_anti_aliased : 1;
@@ -58,12 +60,12 @@ extern const BstGlobals * const bst_globals;
 
 /* --- BstGConfig macros --- */
 #define BST_TYPE_GCONFIG              (bst_type_id_BstGConfig)
-#define BST_GCONFIG(object)           (BSE_CHECK_STRUCT_CAST ((object), BST_TYPE_GCONFIG, BstGConfig))
-#define BST_GCONFIG_CLASS(class)      (BSE_CHECK_CLASS_CAST ((class), BST_TYPE_GCONFIG, BstGConfigClass))
-#define BST_IS_GCONFIG(object)        (BSE_CHECK_STRUCT_TYPE ((object), BST_TYPE_GCONFIG))
-#define BST_IS_GCONFIG_CLASS(class)   (BSE_CHECK_CLASS_TYPE ((class), BST_TYPE_GCONFIG))
-#define BST_GCONFIG_GET_CLASS(object) ((BstGConfigClass*) (((BseObject*) (object))->bse_struct.bse_class))
-extern BseType bst_type_id_BstGConfig;
+#define BST_GCONFIG(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), BST_TYPE_GCONFIG, BstGConfig))
+#define BST_GCONFIG_CLASS(class)      (G_TYPE_CHECK_CLASS_CAST ((class), BST_TYPE_GCONFIG, BstGConfigClass))
+#define BST_IS_GCONFIG(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), BST_TYPE_GCONFIG))
+#define BST_IS_GCONFIG_CLASS(class)   (G_TYPE_CHECK_CLASS_TYPE ((class), BST_TYPE_GCONFIG))
+#define BST_GCONFIG_GET_CLASS(object) (G_TYPE_INSTANCE_GET_CLASS ((object), BST_TYPE_GCONFIG, BstGConfigClass))
+extern GType bst_type_id_BstGConfig;
 
 
 /* --- structures & typedefs --- */
@@ -83,11 +85,11 @@ struct _BstGConfigClass
 
 /* --- prototypes --- */
 void     bst_globals_init               (void);
+void	 bst_globals_set_rc_version	(const gchar *rc_version);
 void	 bst_globals_set_xkb_symbol	(const gchar *xkb_symbol);
 
 
 
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
