@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ static void	bst_item_view_class_init	(BstItemViewClass	*klass);
 static void	bst_item_view_init		(BstItemView		*item_view,
 						 BstItemViewClass	*real_class);
 static void	bst_item_view_destroy		(GtkObject		*object);
-static void	bst_item_view_finalize		(GtkObject		*object);
+static void	bst_item_view_finalize		(GObject		*object);
 
 
 /* --- item clist --- */
@@ -68,7 +68,7 @@ bst_item_view_get_type (void)
 	(GtkClassInitFunc) NULL,
       };
       
-      item_view_type = gtk_type_unique (GTK_TYPE_VPANED, &item_view_info);
+      item_view_type = gtk_type_unique (GTK_TYPE_ALIGNMENT, &item_view_info);
     }
   
   return item_view_type;
@@ -82,10 +82,11 @@ bst_item_view_class_init (BstItemViewClass *class)
   object_class = GTK_OBJECT_CLASS (class);
   
   bst_item_view_class = class;
-  parent_class = gtk_type_class (GTK_TYPE_VPANED);
+  parent_class = gtk_type_class (GTK_TYPE_ALIGNMENT);
   
+  G_OBJECT_CLASS (object_class)->finalize = bst_item_view_finalize;
+
   object_class->destroy = bst_item_view_destroy;
-  object_class->finalize = bst_item_view_finalize;
   
   class->n_ops = 0;
   class->ops = NULL;
@@ -99,8 +100,16 @@ static void
 bst_item_view_init (BstItemView      *item_view,
 		    BstItemViewClass *real_class)
 {
+  item_view->paned = gtk_widget_new (GTK_TYPE_VPANED,
+				     "visible", TRUE,
+				     "parent", item_view,
+				     NULL);
+  gtk_widget_ref (item_view->paned);
+
   item_view->item_type = 0;
-  item_view->container = NULL;
+  item_view->container = 0;
+  item_view->id_format = g_strdup ("%03u");
+  item_view->item_list_pos = 0;
   item_view->item_clist = NULL;
   item_view->param_view = NULL;
   item_view->op_widgets = g_new0 (GtkWidget*, real_class->n_ops);
@@ -110,7 +119,7 @@ bst_item_view_init (BstItemView      *item_view,
 static void
 bst_item_view_destroy_contents (BstItemView *item_view)
 {
-  gtk_container_foreach (GTK_CONTAINER (item_view), (GtkCallback) gtk_widget_destroy, NULL);
+  gtk_container_foreach (GTK_CONTAINER (item_view->paned), (GtkCallback) gtk_widget_destroy, NULL);
 }
 
 static void
@@ -124,13 +133,13 @@ bst_item_view_destroy (GtkObject *object)
   
   bst_item_view_destroy_contents (item_view);
   
-  bst_item_view_set_container (item_view, NULL);
+  bst_item_view_set_container (item_view, 0);
   
   GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
 static void
-bst_item_view_finalize (GtkObject *object)
+bst_item_view_finalize (GObject *object)
 {
   BstItemView *item_view;
   
@@ -138,36 +147,52 @@ bst_item_view_finalize (GtkObject *object)
   
   item_view = BST_ITEM_VIEW (object);
   
+  g_free (item_view->id_format);
   g_free (item_view->op_widgets);
   
-  GTK_OBJECT_CLASS (parent_class)->finalize (object);
+  gtk_widget_unref (item_view->paned);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
 bst_item_view_item_changed (BstItemView *item_view,
-			    BseItem	*item)
+			    BswProxy     item)
 {
   GtkCList *clist;
   gint row;
   
   clist = GTK_CLIST (item_view->item_clist);
-  row = gtk_clist_find_row_from_data (clist, item);
+  row = gtk_clist_find_row_from_data (clist, (gpointer) item);
   if (row >= 0)
     {
-      gchar *string;
+      gchar *string, *str;
       
-      string = g_strdup_printf ("%03u", bse_item_get_seqid (BSE_ITEM (item)));
-      gtk_clist_set_text (clist, row, CLIST_SEQID, string);
+      string = g_strdup_printf (item_view->id_format, bse_item_get_seqid (bse_object_from_id (item)));
+      str = NULL;
+      if (!gtk_clist_get_text (clist, row, CLIST_SEQID, &str) || strcmp (str, string))
+	gtk_clist_set_text (clist, row, CLIST_SEQID, string);
+      g_free (string);
+      string = bsw_item_get_name (item);
+      if (!string)
+	string = "";
+      str = NULL;
+      if (!gtk_clist_get_text (clist, row, CLIST_NAME, &str) || strcmp (str, string))
+	gtk_clist_set_text (clist, row, CLIST_NAME, string);
+      string = NULL;
+      g_object_get (bse_object_from_id (item), "blurb", &string, NULL);
+      if (!string)
+	string = g_strdup ("");
+      if (!gtk_clist_get_text (clist, row, CLIST_BLURB, &str) || strcmp (str, string))
+	gtk_clist_set_text (clist, row, CLIST_BLURB, string);
       g_free (string);
-      gtk_clist_set_text (clist, row, CLIST_NAME, BSE_OBJECT_NAME (item));
-      gtk_clist_set_text (clist, row, CLIST_BLURB, bse_object_get_blurb (BSE_OBJECT (item)));
     }
 }
 
 static void
-bst_item_view_item_param_changed (BstItemView  *item_view,
-				  BseParamSpec *pspec,
-				  BseItem      *item)
+bst_item_view_item_param_changed (BstItemView *item_view,
+				  GParamSpec  *pspec,
+				  BswProxy     item)
 {
   bst_item_view_item_changed (item_view, item);
 }
@@ -175,34 +200,35 @@ bst_item_view_item_param_changed (BstItemView  *item_view,
 static void
 bst_item_view_build_param_view (BstItemView *item_view)
 {
-  BseContainer *container = item_view->container;
-  GList *list, *free_list;
-  BseItem *item = NULL;
+  BswProxy container = item_view->container;
+  BswProxy item = 0;
+  BswIterProxy *iter;
   
   g_return_if_fail (item_view->param_view == NULL);
   
-  free_list = bse_container_list_items (container);
-  for (list = free_list; list; list = list->next)
-    if (bse_type_is_a (BSE_OBJECT_TYPE (list->data), item_view->item_type))
-      item = list->data;
-  g_list_free (free_list);
-  if (BSE_IS_ITEM (item))
+  for (iter = bsw_container_list_items (container); bsw_iter_n_left (iter); bsw_iter_next (iter))
+    if (g_type_is_a (bsw_proxy_type (bsw_iter_get_proxy (iter)), item_view->item_type))
+      item = bsw_iter_get_proxy (iter);
+  bsw_iter_free (iter);
+
+  if (BSW_IS_ITEM (item))
     {
       gint default_param_view_height = BST_ITEM_VIEW_GET_CLASS (item_view)->default_param_view_height;
 
-      item_view->param_view = bst_param_view_new (BSE_OBJECT (item));
-      gtk_widget_set (item_view->param_view,
-		      "signal::destroy", gtk_widget_destroyed, &item_view->param_view,
-		      "visible", TRUE,
-		      default_param_view_height > 0 ? "height" : NULL, default_param_view_height,
-		      NULL);
-      gtk_paned_pack2 (GTK_PANED (item_view),
-		       item_view->param_view,
-		       TRUE,
-		       TRUE);
-	
+      item_view->param_view = bst_param_view_new (item);
+      g_object_set (item_view->param_view,
+		    "visible", TRUE,
+		    default_param_view_height > 0 ? "height_request" : NULL, default_param_view_height,
+		    NULL);
+      g_object_connect (item_view->param_view,
+			"signal::destroy", gtk_widget_destroyed, &item_view->param_view,
+			NULL);
+      (item_view->item_list_pos ? gtk_paned_pack1 : gtk_paned_pack2) (GTK_PANED (item_view->paned),
+								      item_view->param_view,
+								      TRUE,
+								      TRUE);
       bst_param_view_set_object (BST_PARAM_VIEW (item_view->param_view),
-				 (BseObject*) bst_item_view_get_current (item_view));
+				 bst_item_view_get_current (item_view));
     }
 }
 
@@ -211,24 +237,19 @@ bst_item_view_item_added (BstItemView  *item_view,
 			  BseItem      *item,
 			  BseContainer *container)
 {
-  if (BSE_IS_ITEM (item) && bse_type_is_a (BSE_OBJECT_TYPE (item), item_view->item_type))
+  if (BSE_IS_ITEM (item) && g_type_is_a (BSE_OBJECT_TYPE (item), item_view->item_type))
     {
       static gchar *text[CLIST_N_COLUMNS] = { 0, };
       gint row;
       GtkCList *clist = GTK_CLIST (item_view->item_clist);
       
-      bse_object_add_data_notifier (item,
-				    "seqid_changed",
-				    bst_item_view_item_changed,
-				    item_view);
-      bse_object_add_data_notifier (item,
-				    "param_changed",
-				    bst_item_view_item_param_changed,
-				    item_view);
-      
+      bsw_proxy_connect (BSE_OBJECT_ID (item),
+			 "swapped_signal::seqid_changed", bst_item_view_item_changed, item_view,
+			 "swapped_signal::notify", bst_item_view_item_param_changed, item_view,
+			 NULL);
       row = gtk_clist_insert (clist, -1, text);
-      gtk_clist_set_row_data (clist, row, item);
-      bst_item_view_item_changed (item_view, item);
+      gtk_clist_set_row_data (clist, row, (gpointer) BSE_OBJECT_ID (item));
+      bst_item_view_item_changed (item_view, BSE_OBJECT_ID (item));
       
       if (!item_view->param_view)
 	bst_item_view_build_param_view (item_view);
@@ -242,19 +263,17 @@ bst_item_view_item_removed (BstItemView  *item_view,
 			    BseItem	 *item,
 			    BseContainer *container)
 {
-  if (BSE_IS_ITEM (item) && bse_type_is_a (BSE_OBJECT_TYPE (item), item_view->item_type))
+  if (BSE_IS_ITEM (item) && g_type_is_a (BSE_OBJECT_TYPE (item), item_view->item_type))
     {
       gint row;
       GtkCList *clist = GTK_CLIST (item_view->item_clist);
       
-      bse_object_remove_notifiers_by_func (item,
-					   bst_item_view_item_changed,
-					   item_view);
-      bse_object_remove_notifiers_by_func (item,
-					   bst_item_view_item_param_changed,
-					   item_view);
+      bsw_proxy_disconnect (BSE_OBJECT_ID (item),
+			    "any_signal", bst_item_view_item_changed, item_view,
+			    "any_signal", bst_item_view_item_param_changed, item_view,
+			    NULL);
       
-      row = gtk_clist_find_row_from_data (clist, item);
+      row = gtk_clist_find_row_from_data (clist, (gpointer) BSE_OBJECT_ID (item));
       if (row >= 0)
 	gtk_clist_remove (clist, row);
       
@@ -265,118 +284,78 @@ bst_item_view_item_removed (BstItemView  *item_view,
 static void
 bst_item_view_release_container (BstItemView  *item_view)
 {
-  bst_item_view_set_container (item_view, NULL);
+  bst_item_view_set_container (item_view, 0);
 }
 
 void
-bst_item_view_set_container (BstItemView  *item_view,
-			     BseContainer *new_container)
+bst_item_view_set_container (BstItemView *item_view,
+			     BswProxy     new_container)
 {
-  GList *list;
-  BseContainer *container;
+  BswProxy container;
+  BswIterProxy* iter;
   
   g_return_if_fail (BST_IS_ITEM_VIEW (item_view));
   if (new_container)
-    g_return_if_fail (BSE_IS_CONTAINER (new_container));
-  
+    g_return_if_fail (BSW_IS_CONTAINER (new_container));
+
   if (item_view->container)
     {
-      GList *free_list;
-      
       container = item_view->container;
-      item_view->container = NULL;
+      item_view->container = 0;
       
       bst_item_view_destroy_contents (item_view);
       
-      free_list = bse_container_list_items (BSE_CONTAINER (container));
-      for (list = free_list; list; list = list->next)
-	if (bse_type_is_a (BSE_OBJECT_TYPE (list->data), item_view->item_type))
-	  {
-	    bse_object_remove_notifiers_by_func (list->data,
-						 bst_item_view_item_changed,
-						 item_view);
-	    bse_object_remove_notifiers_by_func (list->data,
-						 bst_item_view_item_param_changed,
-						 item_view);
-	  }
-      g_list_free (free_list);
-      
-      bse_object_remove_notifiers_by_func (container,
-					   bst_item_view_item_removed,
-					   item_view);
-      bse_object_remove_notifiers_by_func (container,
-					   bst_item_view_item_added,
-					   item_view);
-      bse_object_remove_notifiers_by_func (container,
-					   bst_item_view_release_container,
-					   item_view);
+      for (iter = bsw_container_list_items (container); bsw_iter_n_left (iter); bsw_iter_next (iter))
+	if (g_type_is_a (bsw_proxy_type (bsw_iter_get_proxy (iter)), item_view->item_type))
+	  bsw_proxy_disconnect (bsw_iter_get_proxy (iter),
+				"any_signal", bst_item_view_item_changed, item_view,
+				"any_signal", bst_item_view_item_param_changed, item_view,
+				NULL);
+      bsw_iter_free (iter);
+
+      g_object_disconnect (bse_object_from_id (container),
+			   "any_signal", bst_item_view_item_removed, item_view,
+			   "any_signal", bst_item_view_item_added, item_view,
+			   "any_signal", bst_item_view_release_container, item_view,
+			   NULL);
     }
 
   if (new_container)
-    g_return_if_fail (BSE_ITEM (new_container)->parent != NULL);
+    g_return_if_fail (bsw_item_get_parent (new_container) != 0);
   
   item_view->container = new_container;
 
   if (item_view->container)
     {
-      GList *free_list;
-      
       container = item_view->container;
       
-      bse_object_add_data_notifier (container,
-				    "set_parent",
-				    bst_item_view_release_container,
-				    item_view);
-      bse_object_add_data_notifier (container,
-				    "item_added",
-				    bst_item_view_item_added,
-				    item_view);
-      bse_object_add_data_notifier (container,
-				    "item_removed",
-				    bst_item_view_item_removed,
-				    item_view);
+      g_object_connect (bse_object_from_id (container),
+			"swapped_signal::set_parent", bst_item_view_release_container, item_view,
+			"swapped_signal::item_added", bst_item_view_item_added, item_view,
+			"swapped_signal::item_removed", bst_item_view_item_removed, item_view,
+			NULL);
+      for (iter = bsw_container_list_items (container); bsw_iter_n_left (iter); bsw_iter_next (iter))
+	if (g_type_is_a (bsw_proxy_type (bsw_iter_get_proxy (iter)), item_view->item_type))
+	  bsw_proxy_connect (bsw_iter_get_proxy (iter),
+			     "swapped_signal::seqid_changed", bst_item_view_item_changed, item_view,
+			     "swapped_signal::notify", bst_item_view_item_param_changed, item_view,
+			     NULL);
+      bsw_iter_free (iter);
 
-      free_list = bse_container_list_items (BSE_CONTAINER (container));
-      for (list = free_list; list; list = list->next)
-	if (bse_type_is_a (BSE_OBJECT_TYPE (list->data), item_view->item_type))
-	  {
-	    bse_object_add_data_notifier (list->data,
-					  "seqid_changed",
-					  bst_item_view_item_changed,
-					  item_view);
-	    bse_object_add_data_notifier (list->data,
-					  "param_changed",
-					  bst_item_view_item_param_changed,
-					  item_view);
-	  }
-      g_list_free (free_list);
-      
       bst_item_view_rebuild (item_view);
     }
 }
 
-static void
-clist_adjust_visibility (GtkCList *clist)
-{
-  if (clist->selection)
-    {
-      gint row = GPOINTER_TO_INT (clist->selection->data);
-      
-      if (gtk_clist_row_is_visible (clist, row) != GTK_VISIBILITY_FULL)
-	gtk_clist_moveto (clist, row, -1, 0.5, 0);
-    }
-}
-
 static void
 bst_item_view_selection_changed (BstItemView *item_view)
 {
   GtkCList *clist = GTK_CLIST (item_view->item_clist);
-  
+
   if (item_view->param_view)
     bst_param_view_set_object (BST_PARAM_VIEW (item_view->param_view),
-			       (BseObject*) bst_item_view_get_current (item_view));
+			       bst_item_view_get_current (item_view));
 
-  clist_adjust_visibility (clist);
+  gtk_clist_moveto_selection (clist);
 }
 
 static void
@@ -391,7 +370,7 @@ button_action (GtkWidget *widget,
 void
 bst_item_view_rebuild (BstItemView *item_view)
 {
-  BseContainer *container;
+  BswProxy container;
   GtkCList *clist;
   GtkWidget *vbox, *list_box;
   guint i;
@@ -410,11 +389,11 @@ bst_item_view_rebuild (BstItemView *item_view)
 			     "border_width", 5,
 			     "visible", TRUE,
 			     NULL);
-  gtk_paned_pack1 (GTK_PANED (item_view),
-		   list_box,
-		   FALSE,
-		   FALSE);
-
+  (item_view->item_list_pos ? gtk_paned_pack2 : gtk_paned_pack1) (GTK_PANED (item_view->paned),
+								  list_box,
+								  FALSE,
+								  FALSE);
+  
   /* action buttons
    */
   vbox = gtk_widget_new (GTK_TYPE_VBOX,
@@ -439,12 +418,13 @@ bst_item_view_rebuild (BstItemView *item_view)
       BstItemViewOp *bop = BST_ITEM_VIEW_GET_CLASS (item_view)->ops + i;
       GtkWidget *label;
 
-      item_view->op_widgets[i] = gtk_widget_new (GTK_TYPE_BUTTON,
-						 "visible", TRUE,
-						 "signal::clicked", button_action, GUINT_TO_POINTER (bop->op),
-						 "signal::destroy", gtk_widget_destroyed, &item_view->op_widgets[i],
-						 "parent", vbox,
-						 NULL);
+      item_view->op_widgets[i] = g_object_connect (gtk_widget_new (GTK_TYPE_BUTTON,
+								   "visible", TRUE,
+								   "parent", vbox,
+								   NULL),
+						   "signal::clicked", button_action, GUINT_TO_POINTER (bop->op),
+						   "signal::destroy", gtk_widget_destroyed, &item_view->op_widgets[i],
+						   NULL);
       label = gtk_widget_new (GTK_TYPE_LABEL,
 			      "visible", TRUE,
 			      "label", bop->op_name,
@@ -455,9 +435,7 @@ bst_item_view_rebuild (BstItemView *item_view)
 			"visible", TRUE,
 			"homogeneous", FALSE,
 			"spacing", 0,
-			"child", bst_forest_from_bse_icon (bst_icon_from_stock (bop->stock_icon),
-							   BST_BUTTON_ICON_WIDTH,
-							   BST_BUTTON_ICON_HEIGHT),
+			"child", bst_image_from_stock (bop->stock_icon, BST_SIZE_BIG_BUTTON),
 			"child", label,
 			"parent", item_view->op_widgets[i],
 			NULL);
@@ -466,24 +444,25 @@ bst_item_view_rebuild (BstItemView *item_view)
   /* item list
    */
   item_view->item_clist =
-    gtk_widget_new (GTK_TYPE_CLIST,
-		    "n_columns", CLIST_N_COLUMNS,
-		    "selection_mode", GTK_SELECTION_BROWSE,
-		    "titles_active", FALSE,
-		    "border_width", 0,
-		    "height", 60,
-		    "signal::destroy", gtk_widget_destroyed, &item_view->item_clist,
-		    "object_signal::select_row", bst_item_view_selection_changed, item_view,
-		    "signal_after::size_allocate", clist_adjust_visibility, NULL,
-		    "signal_after::map", clist_adjust_visibility, NULL,
-		    "visible", TRUE,
-		    "parent", gtk_widget_new (GTK_TYPE_SCROLLED_WINDOW,
-					      "visible", TRUE,
-					      "hscrollbar_policy", GTK_POLICY_AUTOMATIC,
-					      "vscrollbar_policy", GTK_POLICY_AUTOMATIC,
-					      "parent", list_box,
-					      NULL),
-		    NULL);
+    g_object_connect (gtk_widget_new (GTK_TYPE_CLIST,
+				      "n_columns", CLIST_N_COLUMNS,
+				      "selection_mode", GTK_SELECTION_BROWSE,
+				      "titles_active", FALSE,
+				      "border_width", 0,
+				      "height_request", 60,
+				      "visible", TRUE,
+				      "parent", gtk_widget_new (GTK_TYPE_SCROLLED_WINDOW,
+								"visible", TRUE,
+								"hscrollbar_policy", GTK_POLICY_AUTOMATIC,
+								"vscrollbar_policy", GTK_POLICY_AUTOMATIC,
+								"parent", list_box,
+								NULL),
+				      NULL),
+		      "signal::destroy", gtk_widget_destroyed, &item_view->item_clist,
+		      "swapped_signal::select_row", bst_item_view_selection_changed, item_view,
+		      "signal_after::size_allocate", gtk_clist_moveto_selection, NULL,
+		      "signal_after::map", gtk_clist_moveto_selection, NULL,
+		      NULL);
   clist = GTK_CLIST (item_view->item_clist);
   gtk_clist_set_column_title (clist, CLIST_SEQID, clist_titles[CLIST_SEQID]);
   gtk_clist_set_column_title (clist, CLIST_NAME, clist_titles[CLIST_NAME]);
@@ -503,11 +482,10 @@ bst_item_view_rebuild (BstItemView *item_view)
 void
 bst_item_view_update (BstItemView *item_view)
 {
-  GList *list;
+  BswProxy container;
   GtkCList *clist;
-  BseContainer *container;
-  GList *free_list;
-  
+  BswIterProxy *iter;
+
   g_return_if_fail (BST_IS_ITEM_VIEW (item_view));
   
   container = item_view->container;
@@ -516,18 +494,21 @@ bst_item_view_update (BstItemView *item_view)
   gtk_clist_freeze (clist);
   gtk_clist_clear (clist);
   
-  free_list = bse_container_list_items (BSE_CONTAINER (container));
-  for (list = free_list; list; list = list->next)
-    if (bse_type_is_a (BSE_OBJECT_TYPE (list->data), item_view->item_type))
-      {
-	static gchar *text[CLIST_N_COLUMNS] = { NULL, };
-	gint row;
-	
-	row = gtk_clist_insert (clist, 0, text);
-	gtk_clist_set_row_data (clist, row, list->data);
-	bst_item_view_item_changed (item_view, list->data);
-      }
-  g_list_free (free_list);
+  for (iter = bsw_container_list_items (container); bsw_iter_n_left (iter); bsw_iter_next (iter))
+    {
+      BswProxy item = bsw_iter_get_proxy (iter);
+
+      if (g_type_is_a (bsw_proxy_type (item), item_view->item_type))
+	{
+	  static gchar *text[CLIST_N_COLUMNS] = { NULL, };
+	  gint row;
+	  
+	  row = gtk_clist_insert (clist, 0, text);
+	  gtk_clist_set_row_data (clist, row, (gpointer) item);
+	  bst_item_view_item_changed (item_view, item);
+	}
+    }
+  bsw_iter_free (iter);
   
   gtk_clist_thaw (clist);
   
@@ -537,17 +518,17 @@ bst_item_view_update (BstItemView *item_view)
 
 void
 bst_item_view_select (BstItemView *item_view,
-		      BseItem	  *item)
+		      BswProxy	   item)
 {
   GtkCList *clist;
   gint row;
   
   g_return_if_fail (BST_IS_ITEM_VIEW (item_view));
-  g_return_if_fail (BSE_IS_ITEM (item));
-  g_return_if_fail (BSE_ITEM (item)->parent == BSE_ITEM (item_view->container));
+  g_return_if_fail (BSW_IS_ITEM (item));
+  g_return_if_fail (bsw_item_get_parent (item) == item_view->container);
   
   clist = GTK_CLIST (item_view->item_clist);
-  row = gtk_clist_find_row_from_data (clist, item);
+  row = gtk_clist_find_row_from_data (clist, (gpointer) item);
   if (row >= 0)
     {
       gtk_clist_freeze (clist);
@@ -558,25 +539,36 @@ bst_item_view_select (BstItemView *item_view,
     }
 }
 
-BseItem*
+BswProxy
 bst_item_view_get_current (BstItemView *item_view)
 {
-  BseItem *item = NULL;
+  BswProxy item = 0;
   GtkCList *clist;
   
-  g_return_val_if_fail (BST_IS_ITEM_VIEW (item_view), NULL);
+  g_return_val_if_fail (BST_IS_ITEM_VIEW (item_view), 0);
   
   clist = GTK_CLIST (item_view->item_clist);
   
   if (clist->selection)
-    item = gtk_clist_get_row_data (clist, GPOINTER_TO_INT (clist->selection->data));
+    item = (BswProxy) gtk_clist_get_row_data (clist, GPOINTER_TO_INT (clist->selection->data));
   
   if (item)
-    g_return_val_if_fail (BSE_IS_ITEM (item), NULL);
+    g_return_val_if_fail (BSW_IS_ITEM (item), 0);
   
   return item;
 }
 
+void
+bst_item_view_set_id_format (BstItemView *item_view,
+			     const gchar *id_format)
+{
+  g_return_if_fail (BST_IS_ITEM_VIEW (item_view));
+  g_return_if_fail (id_format != NULL);
+
+  g_free (item_view->id_format);
+  item_view->id_format = g_strdup (id_format);
+}
+
 void
 bst_item_view_operate (BstItemView *item_view,
 		       BstOps	    op)
@@ -627,3 +619,60 @@ bst_item_view_can_operate (BstItemView *item_view,
   
   return can_do;
 }
+
+void
+bst_item_view_set_layout (BstItemView *item_view,
+			  gboolean     horizontal,
+			  guint        pos)
+{
+  GtkWidget *child1, *child2;
+
+  g_return_if_fail (BST_IS_ITEM_VIEW (item_view));
+
+  if (item_view->item_list_pos)
+    {
+      child2 = GTK_PANED (item_view->paned)->child1;
+      child1 = GTK_PANED (item_view->paned)->child2;
+    }
+  else
+    {
+      child1 = GTK_PANED (item_view->paned)->child1;
+      child2 = GTK_PANED (item_view->paned)->child2;
+    }
+  if (child1)
+    {
+      gtk_widget_ref (child1);
+      gtk_container_remove (GTK_CONTAINER (child1->parent), child1);
+    }
+  if (child2)
+    {
+      gtk_widget_ref (child2);
+      gtk_container_remove (GTK_CONTAINER (child2->parent), child2);
+    }
+  gtk_widget_destroy (item_view->paned);
+  gtk_widget_unref (item_view->paned);
+
+  item_view->item_list_pos = pos ? 1 : 0;
+
+  item_view->paned = gtk_widget_new (horizontal ? GTK_TYPE_HPANED : GTK_TYPE_VPANED,
+				     "visible", TRUE,
+				     "parent", item_view,
+				     NULL);
+  gtk_widget_ref (item_view->paned);
+  if (child1)
+    {
+      (item_view->item_list_pos ? gtk_paned_pack2 : gtk_paned_pack1) (GTK_PANED (item_view->paned),
+								      child1,
+								      FALSE,
+								      FALSE);
+      gtk_widget_unref (child1);
+    }
+  if (child2)
+    {
+      (item_view->item_list_pos ? gtk_paned_pack1 : gtk_paned_pack2) (GTK_PANED (item_view->paned),
+								      child2,
+								      TRUE,
+								      TRUE);
+      gtk_widget_unref (child2);
+    }
+}
