@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
 
@@ -34,7 +33,7 @@ extern "C" {
 #define	BST_ITEM_VIEW_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), BST_TYPE_ITEM_VIEW, BstItemViewClass))
 #define	BST_IS_ITEM_VIEW(object)      (GTK_CHECK_TYPE ((object), BST_TYPE_ITEM_VIEW))
 #define	BST_IS_ITEM_VIEW_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_ITEM_VIEW))
-#define BST_ITEM_VIEW_GET_CLASS(obj)  ((BstItemViewClass*) (((GtkObject*) (obj))->klass))
+#define BST_ITEM_VIEW_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), BST_TYPE_ITEM_VIEW, BstItemViewClass))
 
 
 /* --- structures & typedefs --- */
@@ -43,11 +42,16 @@ typedef	struct	_BstItemViewClass	BstItemViewClass;
 typedef	struct	_BstItemViewOp		BstItemViewOp;
 struct _BstItemView
 {
-  GtkVPaned	 parent_object;
+  GtkAlignment	 parent_object;
 
-  BseType	 item_type;
+  GtkWidget	*paned;
+
+  GType  	 item_type;
   
-  BseContainer	*container;
+  BswProxy	 container;
+
+  gchar		*id_format;
+  guint		 item_list_pos;
   
   GtkWidget	*item_clist;
   
@@ -57,12 +61,12 @@ struct _BstItemView
 };
 struct _BstItemViewClass
 {
-  GtkVPanedClass parent_class;
+  GtkAlignmentClass parent_class;
 
-  guint			 n_ops;
-  BstItemViewOp		*ops;
+  guint		    n_ops;
+  BstItemViewOp	   *ops;
 
-  guint                  default_param_view_height;
+  guint             default_param_view_height;
 
   void          (*operate)       (BstItemView	*item_view,
 				  BstOps	 op);
@@ -71,9 +75,9 @@ struct _BstItemViewClass
 };
 struct _BstItemViewOp
 {
-  gchar	*op_name;
-  guint	 op;
-  guint  stock_icon;
+  gchar	      *op_name;
+  guint	       op;
+  const gchar *stock_icon;
 };
 
 
@@ -82,19 +86,23 @@ GtkType		bst_item_view_get_type		(void);
 void		bst_item_view_rebuild		(BstItemView	*item_view);
 void		bst_item_view_update		(BstItemView	*item_view);
 void		bst_item_view_select		(BstItemView	*item_view,
-						 BseItem	*item);
-BseItem*	bst_item_view_get_current	(BstItemView	*item_view);
+						 BswProxy	 item);
+BswProxy	bst_item_view_get_current	(BstItemView	*item_view);
 void		bst_item_view_set_container	(BstItemView	*item_view,
-						 BseContainer	*new_container);
+						 BswProxy	 new_container);
 void		bst_item_view_operate		(BstItemView	*item_view,
 						 BstOps		 op);
 gboolean	bst_item_view_can_operate	(BstItemView	*item_view,
 						 BstOps		 op);
+void		bst_item_view_set_id_format	(BstItemView	*item_view,
+						 const gchar	*id_format);
+void		bst_item_view_set_layout	(BstItemView	*item_view,
+						 gboolean	 horizontal,
+						 guint		 pos);
 
 
 
 #ifdef __cplusplus
-#pragma {
 }
 #endif /* __cplusplus */
 
