@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,19 +18,19 @@
 #include        "bstdefs.h"
 
 #include        "bstapp.h"
-#include        "bstsamplerepo.h"
+#include        "bstsplash.h"
 #include	"bstxkb.h"
 #include	"bstkeytables.h"
 #include	"bstgconfig.h"
+#include	"bstusermessage.h"
+#include	"bststatusbar.h"
 #include	"bstpreferences.h"
+#include	"bsttexttools.h"
 #include	"../PKG_config.h"
-
-
-
-
-#define	PROGRAM	"BEAST"
-#define	TITLE	"Beast"
-#define	VERSION	"Pre-Alpha"
+#include	"images/beast-images.h"
+#include	<unistd.h>
+#include	<stdio.h>
+#include	<string.h>
 
 
 /* --- prototypes --- */
@@ -42,14 +42,15 @@ static void			bst_print_blurb		(FILE	     *fout,
 
 
 /* --- variables --- */
-static guint        args_changed_signal_id = 0;
 BstDebugFlags       bst_debug_flags = 0;
+gboolean            bst_dvl_hints = FALSE;
 static GDebugKey    bst_debug_keys[] = { /* keep in sync with bstdefs.h */
   { "keytable",		BST_DEBUG_KEYTABLE, },
   { "samples",		BST_DEBUG_SAMPLES, },
 };
 static const guint  bst_n_debug_keys = sizeof (bst_debug_keys) / sizeof (bst_debug_keys[0]);
 static gboolean     arg_force_xkb = FALSE;
+static gboolean     bst_load_plugins = TRUE;
 static const gchar *bst_rc_string =
 ( "style'BstTooltips-style'"
   "{"
@@ -60,39 +61,85 @@ static const gchar *bst_rc_string =
 
 
 /* --- functions --- */
+static void
+gtk_lock (gpointer null)
+{
+  GDK_THREADS_ENTER ();
+}
+static void
+gtk_unlock (gpointer null)
+{
+  GDK_THREADS_LEAVE ();
+}
+
+static void
+splash_update_item (gpointer     data,
+		    const gchar *item)
+{
+  bst_splash_update_item (data, "%s", item);
+}
+
 int
 main (int   argc,
       char *argv[])
 {
-  BsePcmDevice *pdev = NULL;
+  BswLockFuncs lfuncs = { NULL, gtk_lock, gtk_unlock };
+  GdkPixbufAnimation *anim;
+  GtkWidget *splash;
   BstApp *app = NULL;
+  gchar *string;
   guint i;
   
-  /* initialize BSE and preferences
+  /* initialize BSE, BSW and preferences
    */
-  bse_init (&argc, &argv);
+  if (0)
+    g_mem_set_vtable (glib_mem_profiler_table);
+  g_thread_init (NULL);
+  g_type_init ();
+  bsw_init (&argc, &argv, &lfuncs);
   bst_globals_init ();
-  
-  /* initialize GUI libraries and pre-parse args
+
+  /* pre-parse BEAST args
    */
   bst_parse_args (&argc, &argv);
-
   g_message ("BEAST: pid = %u", getpid ());
-  
+
+  /* initialize GUI libraries and patch them up ;)
+   */
   gtk_init (&argc, &argv);
+  g_set_prgname ("BEAST");
+  gtk_post_init_patch_ups ();
+
+  GDK_THREADS_ENTER ();
+
+  /* first, popup splash screen
+   */
+  splash = bst_splash_new ("BEAST startup", BST_SPLASH_WIDTH, BST_SPLASH_HEIGHT, 15);
+  bst_splash_set_text (splash,
+		       "<b><big>BEAST</big></b>\n"
+		       "<b>The Bedevilled Audio System</b>\n"
+		       "<b>Version %s (%s)</b>\n",
+		       BST_VERSION, BST_VERSION_HINT);
+  bst_splash_update_entity (splash, "Startup");
+  bst_splash_show_now (splash);
+
+  /* BEAST initialization
+   */
+  _bst_utils_init ();
+  bst_splash_update_item (splash, "Objects");
+  bst_init_gentypes ();
   gtk_rc_parse_string (bst_rc_string);
-  gdk_rgb_init ();
-  gnome_type_init ();
-  // gnome_init (PROGRAM, VERSION, argc, argv);
-  bst_free_radio_button_get_type ();
+  g_type_name (bst_free_radio_button_get_type ());	/* urg, GCC_CONST */
   
-  /* parse rc file */
-  if (1)
+  /* parse rc file
+   */
+  if (TRUE)
     {
       BseGConfig *gconf;
       BseErrorType error;
       gchar *file_name;
       
+      bst_splash_update_item (splash, "Rc File");
       file_name = BST_STRDUP_RC_FILE ();
       gconf = bse_object_new (BST_TYPE_GCONFIG, NULL);
       bse_gconfig_revert (gconf);
@@ -100,148 +147,180 @@ main (int   argc,
       if (error != BSE_ERROR_FILE_NOT_FOUND)
 	{
 	  bse_gconfig_apply (gconf);
+#if 0
 	  if (error)
-	    g_warning ("error parsing rc-file \"%s\": %s", file_name, bse_error_blurb (error));
+	    g_printerr ("BEAST: error parsing rc-file \"%s\": %s\n", file_name, bse_error_blurb (error));
+#endif
 	}
       bse_object_unref (BSE_OBJECT (gconf));
       g_free (file_name);
     }
-  
+
+  /* show splash images
+   */
+  bst_splash_update_item (splash, "Splash Image");
+  string = g_strconcat (BST_PATH_IMAGES, G_DIR_SEPARATOR_S, BST_SPLASH_IMAGE, NULL);
+  anim = gdk_pixbuf_animation_new_from_file (string, NULL);
+  g_free (string);
+  bst_splash_update ();
+  if (anim)
+    {
+      bst_splash_set_animation (splash, anim);
+      g_object_unref (anim);
+    }
+
+  /* register dynamic types and modules (plugins)
+   */
+  bst_splash_update_entity (splash, "Plugins");
+  if (bst_load_plugins)
+    bsw_register_plugins (NULL, TRUE, NULL, splash_update_item, splash);
+  bst_text_add_tsm_path (BST_PATH_DOCS);
+  bst_text_add_tsm_path (BST_PATH_IMAGES);
+  bst_text_add_tsm_path (".");
+
+  /* debugging hook
+   */
+  string = g_getenv ("BEAST_SLEEP4GDB");
+  if (string && atoi (string) > 0)
+    {
+      g_message ("going into sleep mode due to debugging request (pid=%u)", getpid ());
+      g_usleep (2147483647);
+    }
+
+  /* register scripts
+   */
+  if (bst_load_plugins)
+    {
+      guint n_scripts;
+
+      bst_splash_update_entity (splash, "Scripts");
+
+      /* script registration, this is done asyncronously,
+       * so we wait until all are done
+       */
+      n_scripts = bsw_server_n_scripts (BSW_SERVER);
+      bsw_register_scripts (NULL, TRUE, NULL, splash_update_item, splash);
+      while (bsw_server_n_scripts (BSW_SERVER) > n_scripts)
+	{
+	  GDK_THREADS_LEAVE ();
+	  g_main_iteration (TRUE);
+	  GDK_THREADS_ENTER ();
+	}
+    }
+
+  bst_splash_update_entity (splash, "Misc");
+
+  /* listen to BseServer notification
+   */
+  bst_splash_update_entity (splash, "Dialogs");
+  bst_user_messages_listen ();
+  bst_status_bar_listen_exec_status ();
+
   /* setup default keytable for pattern editor class
    */
+  bst_splash_update_entity (splash, "Keytable");
   bst_key_table_install_patch (bst_key_table_from_xkb (gdk_get_display ()));
-  
-  
-  /* check load BSE plugins to register types
+
+  /* hide splash screen, but grav events until we're done
    */
-  if (1)
-    {
-      GList *free_list, *list;
-      
-      free_list = bse_plugin_dir_list_files (BSE_PATH_PLUGINS);
-      for (list = free_list; list; list = list->next)
-	{
-	  gchar *error, *string = list->data;
-	  
-	  g_message ("loading plugin \"%s\"...", string);
-	  error = bse_plugin_check_load (string);
-	  if (error)
-	    g_warning ("error encountered loading plugin \"%s\": %s", string, error);
-	  g_free (string);
-	}
-      g_list_free (free_list);
-      if (!free_list)
-	g_warning ("strange, can't find any plugins, please check %s", BSE_PATH_PLUGINS);
-    }
-  
-  
-  /* hackery rulez!
-   */
-  args_changed_signal_id =
-    gtk_object_class_user_signal_new (gtk_type_class (GTK_TYPE_OBJECT),
-				      "args_changed",
-				      GTK_RUN_ACTION,
-				      gtk_signal_default_marshaller,
-				      GTK_TYPE_NONE, 0);
-  
-  
-  /* setup PCM Devices
-   */
-  if (!pdev && BSE_TYPE_ID (BsePcmDeviceAlsa) && !BST_DISABLE_ALSA)
-    {
-      BseErrorType error;
-      
-      pdev = (BsePcmDevice*) bse_object_new (BSE_TYPE_ID (BsePcmDeviceAlsa), NULL); /* FIXME: TYPE_ID */
-      error = bse_pcm_device_update_caps (pdev);
-      if (error && error != BSE_ERROR_DEVICE_BUSY)
-	{
-	  bse_object_unref (BSE_OBJECT (pdev));
-	  pdev = NULL;
-	}
-    }
-  if (!pdev && BSE_TYPE_ID (BsePcmDeviceOSS))
-    {
-      BseErrorType error;
-      
-      pdev = (BsePcmDevice*) bse_object_new (BSE_TYPE_ID (BsePcmDeviceOSS), NULL); /* FIXME: TYPE_ID */
-      error = bse_pcm_device_update_caps (pdev);
-      if (error && error != BSE_ERROR_DEVICE_BUSY)
-	{
-	  bse_object_unref (BSE_OBJECT (pdev));
-	  pdev = NULL;
-	}
-    }
-  if (!pdev)
-    g_error ("No PCM device driver known");
-  bse_heart_register_device ("Master", pdev);
-  bse_object_unref (BSE_OBJECT (pdev));
-  bse_heart_set_default_odevice ("Master");
-  bse_heart_set_default_idevice ("Master");
-  
-  
-  /* register sample repositories
-   */
-  bst_sample_repo_init ();
-  
-  
+  gtk_widget_hide (splash);
+  gtk_grab_add (splash);
+
   /* open files given on command line
    */
   for (i = 1; i < argc; i++)
     {
-      BseStorage *storage = bse_storage_new ();
-      BseErrorType error;
-      
-      error = bse_storage_input_file (storage, argv[i]);
+      BswProxy project, wrepo;
+      BswErrorType error;
+
+      bst_splash_update ();
+
+      /* load waves into the last project */
+      if (bsw_server_can_load (BSW_SERVER, argv[i]))
+	{
+	  if (!app)
+	    {
+	      project = bsw_server_use_new_project (BSW_SERVER, "Untitled.bse");
+	      wrepo = bsw_project_ensure_wave_repo (project);
+	      error = bsw_wave_repo_load_file (wrepo, argv[i]);
+	      if (!error)
+		{
+		  app = bst_app_new (project);
+		  gtk_idle_show_widget (GTK_WIDGET (app));
+		  bsw_item_unuse (project);
+		  continue;
+		}
+	      bsw_item_unuse (project);
+	    }
+	  else
+	    {
+	      BswProxy wrepo = bsw_project_ensure_wave_repo (app->project);
+	      
+	      bst_status_printf (BST_STATUS_WAIT, NULL, "Loading \"%s\"", argv[i]);
+	      error = bsw_wave_repo_load_file (wrepo, argv[i]);
+	      bst_status_eprintf (error, "Loading \"%s\"", argv[i]);
+	      if (!error)
+		continue;
+	    }
+	}
+      project = bsw_server_use_new_project (BSW_SERVER, argv[i]);
+      error = bsw_project_restore_from_file (project, argv[i]);
       
       if (!error)
 	{
-	  BseProject *project = bse_project_new (argv[i]);
-	  
-	  bse_storage_set_path_resolver (storage, bse_project_path_resolver, project);
-	  error = bse_project_restore (project, storage);
-	  if (!error)
-	    {
-	      app = bst_app_new (project);
-	      gtk_idle_show_widget (GTK_WIDGET (app));
-	    }
-	  bse_object_unref (BSE_OBJECT (project));
+	  app = bst_app_new (project);
+	  gtk_idle_show_widget (GTK_WIDGET (app));
 	}
-      bse_storage_destroy (storage);
+      bsw_item_unuse (project);
+      
       if (error)
-	g_warning ("failed to load project `%s': %s", /* FIXME */
-		   argv[i],
-		   bse_error_blurb (error));
+	bst_status_eprintf (error, "Loading project \"%s\"", argv[i]);
     }
-  
+
   /* open default app window
    */
   if (!app)
     {
-      BseProject *project = bse_project_new ("Untitled.bse");
+      BswProxy project = bsw_server_use_new_project (BSW_SERVER, "Untitled.bse");
       
+      bsw_project_ensure_wave_repo (project);
       app = bst_app_new (project);
-      bse_object_unref (BSE_OBJECT (project));
-      /* bst_app_operate (app, BST_OP_PROJECT_NEW_SONG); */
+      bsw_item_unuse (project);
       gtk_idle_show_widget (GTK_WIDGET (app));
     }
   
-  
-  
-  /* and away into the main loop
+  /* fire up release notes dialog
    */
+  if (!bst_globals->rc_version || strcmp (bst_globals->rc_version, BST_VERSION))
+    {
+      bst_app_operate (app, BST_OP_HELP_RELEASE_NOTES);
+      bst_globals_set_rc_version (BST_VERSION);
+    }
+
+  /* destroy splash to release grabs,
+   * away into the main loop
+   */
+  gtk_widget_destroy (splash);
   gtk_main ();
   
-  /* stop everything playing */
-  bse_heart_reset_all_attach ();
+  /* stop everything playing
+   */
+  // bse_heart_reset_all_attach ();
+
+  /* take down GUI leftovers
+   */
+  bst_user_messages_kill ();
   
-  /* FXME: wrt cleanup cycles */
-  
-  /* perform necessary cleanup cycles */
+  /* perform necessary cleanup cycles
+   */
+  GDK_THREADS_LEAVE ();
   while (g_main_iteration (FALSE))
     ;
+  GDK_THREADS_ENTER ();
   
-  /* save rc file */
-  if (1)
+  /* save rc file
+   */
+  if (TRUE)
     {
       BseGConfig *gconf;
       BseErrorType error;
@@ -257,12 +336,17 @@ main (int   argc,
       g_free (file_name);
     }
   
-  /* remove pcm devices */
-  bse_heart_unregister_all_devices ();
+  /* remove pcm devices
+   */
+  // bse_heart_unregister_all_devices ();
   
-  /* perform necessary cleanup cycles */
+  /* perform necessary cleanup cycles
+   */
+  GDK_THREADS_LEAVE ();
   while (g_main_iteration (FALSE))
     ;
+
+  bse_object_debug_leaks ();
   
   return 0;
 }
@@ -291,7 +375,17 @@ bst_parse_args (int    *argc_p,
   
   for (i = 1; i < argc; i++)
     {
-      if (strcmp ("--beast-debug", argv[i]) == 0 ||
+      if (strcmp ("--no-plugins", argv[i]) == 0)
+	{
+	  bst_load_plugins = FALSE;
+	  argv[i] = NULL;
+	}
+      else if (strcmp ("--beast-developer-extensions", argv[i]) == 0)
+	{
+	  bse_developer_extensions = TRUE;
+	  argv[i] = NULL;
+	}
+      else if (strcmp ("--beast-debug", argv[i]) == 0 ||
 	  strncmp ("--beast-debug=", argv[i], 14) == 0)
 	{
 	  gchar *equal = argv[i] + 13;
@@ -359,6 +453,11 @@ bst_parse_args (int    *argc_p,
 	    }
 	  argv[i] = NULL;
 	}
+      else if (strcmp ("--hints", argv[i]) == 0)
+	{
+	  bst_dvl_hints = TRUE;
+          argv[i] = NULL;
+	}
       else if (strcmp ("--force-xkb", argv[i]) == 0)
 	{
 	  arg_force_xkb = TRUE;
@@ -367,17 +466,35 @@ bst_parse_args (int    *argc_p,
       else if (strcmp ("-h", argv[i]) == 0 ||
 	       strcmp ("--help", argv[i]) == 0)
 	{
-	  bst_print_blurb (stderr, TRUE);
+	  bst_print_blurb (stdout, TRUE);
           argv[i] = NULL;
 	  exit (0);
 	}
       else if (strcmp ("-v", argv[i]) == 0 ||
 	       strcmp ("--version", argv[i]) == 0)
 	{
-	  bst_print_blurb (stderr, FALSE);
+	  bst_print_blurb (stdout, FALSE);
 	  argv[i] = NULL;
 	  exit (0);
 	}
+      else if (strcmp ("--print-path", argv[i]) == 0 ||
+	       strncmp ("--print-path=", argv[i], 13) == 0)
+	{
+	  gchar *arg = argv[i][13 - 1] == '=' ? argv[i] + 13 : (argv[i + 1] ? argv[i + 1] : "");
+	  if (strcmp (arg, "docs") == 0)
+	    fprintf (stdout, "%s\n", BST_PATH_DOCS);
+	  else if (strcmp (arg, "images") == 0)
+	    fprintf (stdout, "%s\n", BST_PATH_IMAGES);
+	  else if (strcmp (arg, "plugins") == 0)
+	    fprintf (stdout, "%s\n", BSE_PATH_PLUGINS);
+	  else if (strcmp (arg, "scripts") == 0)
+	    fprintf (stdout, "%s\n", BSW_PATH_SCRIPTS);
+	  else if (strcmp (arg, "samples") == 0)
+	    fprintf (stdout, "%s\n", BST_PATH_DATA_SAMPLES);
+	  else
+	    g_message ("no such resource path: %s", arg);
+	  exit (0);
+	}
     }
   
   e = 0;
@@ -418,9 +535,11 @@ bst_print_blurb (FILE    *fout,
       fprintf (fout, "\n");
       fprintf (fout, "Compiled for: %s\n", BST_ARCH_NAME);
       fprintf (fout, "\n");
-      fprintf (fout, "Doc-path:    %s\n", BST_PATH_DOCS);
-      fprintf (fout, "Plugin-path: %s\n", BSE_PATH_PLUGINS);
-      fprintf (fout, "Sample-path: %s\n", BST_PATH_DATA_SAMPLES);
+      fprintf (fout, "Doc Path:    %s\n", BST_PATH_DOCS);
+      fprintf (fout, "Image Path:  %s\n", BST_PATH_IMAGES);
+      fprintf (fout, "Plugin Path: %s\n", BSE_PATH_PLUGINS);
+      fprintf (fout, "Script Path: %s\n", BSW_PATH_SCRIPTS);
+      fprintf (fout, "Sample Path: %s\n", BST_PATH_DATA_SAMPLES);
       fprintf (fout, "\n");
       fprintf (fout, "BEAST comes with ABSOLUTELY NO WARRANTY.\n");
       fprintf (fout, "You may redistribute copies of BEAST under the terms of\n");
@@ -431,22 +550,27 @@ bst_print_blurb (FILE    *fout,
   else
     {
       fprintf (fout, "Usage: beast [options] [files...]\n");
-      fprintf (fout, "  --beast-debug=keys		enable certain BEAST debug stages\n");
-      fprintf (fout, "  --beast-no-debug=keys		disable certain BEAST debug stages\n");
-      fprintf (fout, "  --force-xkb			force XKB keytable queries\n");
-      fprintf (fout, "  --bse-debug=keys		enable certain BSE debug stages\n");
-      fprintf (fout, "  --bse-no-debug=keys		disable certain BSE debug stages\n");
-      fprintf (fout, "  -h, --help			show this help message\n");
-      fprintf (fout, "  -v, --version			print version informations\n");
-      fprintf (fout, "  --display=DISPLAY		X server to contact; see X(1)\n");
-      fprintf (fout, "  --no-xshm			disable use of X shared memory extension\n");
-      fprintf (fout, "  --gtk-debug=FLAGS		Gtk+ debugging flags to enable\n");
-      fprintf (fout, "  --gtk-no-debug=FLAGS		Gtk+ debugging flags to disable\n");
-      fprintf (fout, "  --gtk-module=MODULE		load additional Gtk+ modules\n");
-      fprintf (fout, "  --gdk-debug=FLAGS		Gdk debugging flags to enable\n");
-      fprintf (fout, "  --gdk-no-debug=FLAGS		Gdk debugging flags to disable\n");
-      fprintf (fout, "  --g-fatal-warnings		make warnings fatal (abort)\n");
-      fprintf (fout, "  --sync   			do all X calls synchronously\n");
+      fprintf (fout, "  --hints                         enrich the GUI with hints usefull for (script) developers\n");
+      fprintf (fout, "  --no-plugins                    disable plugins (debug usage only)\n");
+      fprintf (fout, "  --force-xkb                     force XKB keytable queries\n");
+      fprintf (fout, "  --beast-debug=keys              enable certain BEAST debug stages\n");
+      fprintf (fout, "  --beast-no-debug=keys           disable certain BEAST debug stages\n");
+      fprintf (fout, "  --bse-debug=keys                enable certain BSE debug stages\n");
+      fprintf (fout, "  --bse-no-debug=keys             disable certain BSE debug stages\n");
+      fprintf (fout, "  --print-path=resource           print the file path for a specific resource\n");
+      fprintf (fout, "  -h, --help                      show this help message\n");
+      fprintf (fout, "  -v, --version                   print version and file paths\n");
+      fprintf (fout, "  --display=DISPLAY               X server for the GUI; see X(1)\n");
+      if (!BST_VERSION_STABLE)
+        {
+          fprintf (fout, "  --gtk-debug=FLAGS               Gtk+ debugging flags to enable\n");
+          fprintf (fout, "  --gtk-no-debug=FLAGS            Gtk+ debugging flags to disable\n");
+          fprintf (fout, "  --gtk-module=MODULE             load additional Gtk+ modules\n");
+          fprintf (fout, "  --gdk-debug=FLAGS               Gdk debugging flags to enable\n");
+          fprintf (fout, "  --gdk-no-debug=FLAGS            Gdk debugging flags to disable\n");
+          fprintf (fout, "  --g-fatal-warnings              make warnings fatal (abort)\n");
+          fprintf (fout, "  --sync                          do all X calls synchronously\n");
+        }
     }
 }
 
@@ -538,7 +662,7 @@ bst_key_table_from_xkb (const gchar *display)
 	g_message ("Failed to find keytable \"%s\"", BST_XKB_SYMBOL);
     }
   
-  if (!patch)
+  if (!patch && 0 /* FIXME: we don't need xkb keytables currently */)
     {
       gchar *xkb_symbol = NULL;
       
@@ -575,72 +699,3 @@ bst_key_table_from_xkb (const gchar *display)
   
   return patch;
 }
-
-/* read bstdefs.h on this */
-void
-bst_update_can_operate (GtkWidget *widget)
-{
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-  
-  /* FIXME, we need to queue a high prioritized idle here as
-   * this function can be called multiple times in a row
-   */
-  
-  /* figure toplevel app, and update it
-   */
-  widget = gtk_widget_get_ancestor (widget, BST_TYPE_APP);
-  g_return_if_fail (BST_IS_APP (widget));
-  
-  bst_app_update_can_operate (BST_APP (widget));
-}
-
-/* read bstdefs.h on this */
-extern void
-bst_object_set (gpointer     object,
-		const gchar *first_arg_name,
-		...)
-{
-  va_list args;
-  
-  g_return_if_fail (GTK_IS_OBJECT (object));
-  
-  gtk_object_ref (object);
-  
-  va_start (args, first_arg_name);
-  
-  if (GNOME_IS_CANVAS_ITEM (object))
-    gnome_canvas_item_set_valist (object, first_arg_name, args);
-  else
-    {
-      GSList *arg_list = NULL;
-      GSList *info_list = NULL;
-      gchar *error;
-      
-      error = gtk_object_args_collect (GTK_OBJECT_TYPE (object),
-				       &arg_list,
-				       &info_list,
-				       first_arg_name,
-				       args);
-      
-      if (error)
-	{
-	  g_warning ("bst_object_set(): %s", error);
-	  g_free (error);
-	}
-      else if (arg_list)
-	{
-	  GSList *arg;
-	  GSList *info;
-	  
-	  for (arg = arg_list, info = info_list; arg; arg = arg->next, info = info->next)
-	    gtk_object_arg_set (object, arg->data, info->data);
-	  
-	  gtk_args_collect_cleanup (arg_list, info_list);
-	}
-    }
-  va_end (args);
-  
-  BST_OBJECT_ARGS_CHANGED (object);
-  
-  gtk_object_unref (object);
-}
