@@ -1,18 +1,18 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Tim Janik and Red Hat, Inc.
+ * Copyright (C) 1998-2002 Tim Janik
  *
- * This library is free software; you can redistribute it and/or
+ * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
+ * Public License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
@@ -24,17 +24,15 @@
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
-
-/* --- Gtk+ type macros --- */
-#define	BST_TYPE_PARAM_VIEW		(bst_param_view_get_type ())
-#define	BST_PARAM_VIEW(object)		(GTK_CHECK_CAST ((object), BST_TYPE_PARAM_VIEW, BstParamView))
-#define	BST_PARAM_VIEW_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), BST_TYPE_PARAM_VIEW, BstParamViewClass))
-#define	BST_IS_PARAM_VIEW(object)	(GTK_CHECK_TYPE ((object), BST_TYPE_PARAM_VIEW))
-#define	BST_IS_PARAM_VIEW_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_PARAM_VIEW))
-#define BST_PARAM_VIEW_GET_CLASS(obj)	((BstParamViewClass*) (((GtkObject*) (obj))->klass))
+/* --- type macros --- */
+#define BST_TYPE_PARAM_VIEW              (bst_param_view_get_type ())
+#define BST_PARAM_VIEW(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), BST_TYPE_PARAM_VIEW, BstParamView))
+#define BST_PARAM_VIEW_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), BST_TYPE_PARAM_VIEW, BstParamViewClass))
+#define BST_IS_PARAM_VIEW(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), BST_TYPE_PARAM_VIEW))
+#define BST_IS_PARAM_VIEW_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), BST_TYPE_PARAM_VIEW))
+#define BST_PARAM_VIEW_GET_CLASS(object) (G_TYPE_INSTANCE_GET_CLASS ((object), BST_TYPE_PARAM_VIEW, BstParamViewClass))
 
 
 /* --- structures & typedefs --- */
@@ -42,35 +40,35 @@ typedef	struct	_BstParamView		BstParamView;
 typedef	struct	_BstParamViewClass	BstParamViewClass;
 struct _BstParamView
 {
-  GtkVWrapBox	 parent_object;
+  GtkVBox	 parent_object;
 
-  BseObject	*object;
+  BswProxy	 object;
 
   GSList	*bparams;
 
-  BseType	 base_type;
-  BseType	 object_type;
+  GType  	 first_base_type;
+  GType  	 last_base_type;
   GPatternSpec  *reject_pattern;
   GPatternSpec  *match_pattern;
-
-  GtkTooltips	*tooltips;
+  GtkWidget	*nil_container;	/* null group */
+  GtkWidget	*container;
 };
 struct _BstParamViewClass
 {
-  GtkVWrapBoxClass parent_class;
+  GtkVBoxClass parent_class;
 };
 
 
 /* --- prototypes --- */
-GtkType		bst_param_view_get_type		(void);
-GtkWidget*	bst_param_view_new		(BseObject	*object);
+GType		bst_param_view_get_type		(void);
+GtkWidget*	bst_param_view_new		(BswProxy	 object);
 void		bst_param_view_update		(BstParamView	*param_view);
 void		bst_param_view_rebuild		(BstParamView	*param_view);
 void		bst_param_view_set_object	(BstParamView	*param_view,
-						 BseObject	*object);
+						 BswProxy	 object);
 void		bst_param_view_set_mask		(BstParamView	*param_view,
-						 BseType	 base_type,
-						 BseType	 param_object_type,
+						 GType  	 first_base_type,
+						 GType  	 last_base_type,
 						 const gchar	*reject_pattern,
 						 const gchar	*match_pattern);
 
@@ -79,7 +77,6 @@ void		bst_param_view_set_mask		(BstParamView	*param_view,
 
 
 #ifdef __cplusplus
-#pragma {
 }
 #endif /* __cplusplus */
 
