@@ -1,24 +1,25 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1999, 2000 Tim Janik and Red Hat, Inc.
+ * Copyright (C) 1999-2002 Tim Janik and Red Hat, Inc.
  *
- * This library is free software; you can redistribute it and/or
+ * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
+ * Public License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 #include "bstpreferences.h"
 
 #include "bstgconfig.h"
+#include <unistd.h>
 
 
 /* --- prototypes --- */
@@ -74,28 +75,29 @@ bst_preferences_class_init (BstPreferencesClass *class)
 static void
 bst_preferences_init (BstPreferences *prefs)
 {
-  GtkWidget *any, *hbox, *button;
-
   prefs->gconf = NULL;
-  prefs->bse_param_view = gtk_widget_new (BST_TYPE_PARAM_VIEW,
-					  "visible", TRUE,
-					  "object_signal::destroy", bse_nullify_pointer, &prefs->bse_param_view,
-					  NULL);
+  prefs->bse_param_view = g_object_connect (gtk_widget_new (BST_TYPE_PARAM_VIEW,
+							    "visible", TRUE,
+							    NULL),
+					    "swapped_signal::destroy", g_nullify_pointer, &prefs->bse_param_view,
+					    NULL);
   bst_param_view_set_mask (BST_PARAM_VIEW (prefs->bse_param_view), BSE_TYPE_GCONFIG, BSE_TYPE_GCONFIG, NULL, NULL);
-  prefs->bst_param_view = gtk_widget_new (BST_TYPE_PARAM_VIEW,
-					  "object_signal::destroy", bse_nullify_pointer, &prefs->bst_param_view,
-					  NULL);
+  prefs->bst_param_view = g_object_connect (gtk_widget_new (BST_TYPE_PARAM_VIEW, NULL),
+					    "swapped_signal::destroy", g_nullify_pointer, &prefs->bst_param_view,
+					    NULL);
   bst_param_view_set_mask (BST_PARAM_VIEW (prefs->bst_param_view), BST_TYPE_GCONFIG, 0, NULL, NULL);
 
-  prefs->notebook = gtk_widget_new (GTK_TYPE_NOTEBOOK,
-				    "visible", TRUE,
-				    "parent", prefs,
-				    "tab_pos", GTK_POS_TOP,
-				    "scrollable", FALSE,
-				    "can_focus", TRUE,
-				    "border_width", 5,
-				    "object_signal::destroy", bse_nullify_pointer, &prefs->notebook,
-				    NULL);
+  prefs->notebook = g_object_connect (gtk_widget_new (GTK_TYPE_NOTEBOOK,
+						      "visible", TRUE,
+						      "parent", prefs,
+						      "tab_pos", GTK_POS_TOP,
+						      "scrollable", FALSE,
+						      "can_focus", TRUE,
+						      "border_width", 5,
+						      NULL),
+				      "swapped_signal::destroy", g_nullify_pointer, &prefs->notebook,
+				      "signal_after::switch-page", gtk_widget_viewable_changed, NULL,
+				      NULL);
   gtk_notebook_append_page (GTK_NOTEBOOK (prefs->notebook), prefs->bst_param_view,
 			    gtk_widget_new (GTK_TYPE_LABEL,
 					    "visible", TRUE,
@@ -106,64 +108,6 @@ bst_preferences_init (BstPreferences *prefs)
 					    "visible", TRUE,
 					    "label", "BSE",
 					    NULL));
-
-  /* dialog bits
-   */
-  any = gtk_widget_new (gtk_hseparator_get_type (),
-			"visible", TRUE,
-			NULL);
-  gtk_box_pack_start (GTK_BOX (prefs), any, FALSE, TRUE, 0);
-  hbox = gtk_widget_new (GTK_TYPE_HBOX,
-			 "homogeneous", TRUE,
-			 "spacing", 5,
-			 "border_width", 5,
-			 "visible", TRUE,
-			 NULL);
-  gtk_box_pack_end (GTK_BOX (prefs), hbox, FALSE, TRUE, 0);
-  button = gtk_widget_new (GTK_TYPE_BUTTON,
-			   "label", "Apply",
-			   "parent", hbox,
-			   "visible", TRUE,
-			   "can_default", TRUE,
-			   "object_signal::clicked", bst_preferences_apply, prefs,
-			   "object_signal::clicked", bst_preferences_save, prefs,
-			   "object_signal::destroy", bse_nullify_pointer, &prefs->apply,
-			   NULL);
-  gtk_tooltips_set_tip (BST_PARAM_VIEW (prefs->bse_param_view)->tooltips, button,
-			"Apply and save the preference values. Some values may only take effect after "
-			"restart. The preference values are locked against modifcation during "
-			"playback.",
-			NULL);
-  prefs->apply = button;
-  button = gtk_widget_new (GTK_TYPE_BUTTON,
-			   "label", "Revert",
-			   "parent", hbox,
-			   "visible", TRUE,
-			   "can_default", TRUE,
-			   "object_signal::clicked", bst_preferences_revert, prefs,
-			   NULL);
-  gtk_tooltips_set_tip (BST_PARAM_VIEW (prefs->bse_param_view)->tooltips, button,
-			"Revert the preference values to the current internal values.",
-			NULL);
-  button = gtk_widget_new (GTK_TYPE_BUTTON,
-			   "label", "Defaults",
-			   "parent", hbox,
-			   "visible", TRUE,
-			   "can_default", TRUE,
-			   "object_signal::clicked", bst_preferences_default_revert, prefs,
-			   NULL);
-  gtk_tooltips_set_tip (BST_PARAM_VIEW (prefs->bse_param_view)->tooltips, button,
-			"Revert to hardcoded default values (factory settings).",
-			NULL);
-  button = gtk_widget_new (GTK_TYPE_BUTTON,
-			   "label", "Close",
-			   "parent", hbox,
-			   "visible", TRUE,
-			   "can_default", TRUE,
-			   "signal::clicked", gtk_toplevel_hide, NULL,
-			   "object_signal::destroy", bse_nullify_pointer, &prefs->close,
-			   NULL);
-  prefs->close = button;
 }
 
 static void
@@ -192,7 +136,8 @@ bst_preferences_new (BseGConfig *gconf)
 static void
 preferences_lock_changed (BstPreferences *prefs)
 {
-  gtk_widget_set_sensitive (prefs->apply, prefs->gconf ? bse_gconfig_can_apply (prefs->gconf) : FALSE);
+  if (prefs->apply)
+    gtk_widget_set_sensitive (prefs->apply, prefs->gconf ? bse_gconfig_can_apply (prefs->gconf) : FALSE);
 }
 
 void
@@ -205,28 +150,27 @@ bst_preferences_set_gconfig (BstPreferences *prefs,
 
   if (prefs->gconf)
     {
-      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bse_param_view), NULL);
-      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bst_param_view), NULL);
-      bse_object_remove_notifiers_by_func (prefs->gconf,
-					   preferences_lock_changed,
-					   prefs);
+      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bse_param_view), 0);
+      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bst_param_view), 0);
+      g_object_disconnect (prefs->gconf,
+			   "any_signal", preferences_lock_changed, prefs,
+			   NULL);
       bse_object_unref (BSE_OBJECT (prefs->gconf));
       prefs->gconf = NULL;
     }
   prefs->gconf = gconf;
-  if (prefs->gconf)
+  if (prefs->gconf && prefs->notebook)
     {
       bse_object_ref (BSE_OBJECT (prefs->gconf));
-      bse_object_add_data_notifier (prefs->gconf,
-				    "lock_changed",
-				    preferences_lock_changed,
-				    prefs);
-      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bse_param_view), BSE_OBJECT (prefs->gconf));
-      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bst_param_view), BSE_OBJECT (prefs->gconf));
-      if (bse_type_next_base (BSE_OBJECT_TYPE (prefs->gconf), BSE_TYPE_GCONFIG))
+      g_object_connect (prefs->gconf,
+			"swapped_signal::lock_changed", preferences_lock_changed, prefs,
+			NULL);
+      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bse_param_view), BSE_OBJECT_ID (prefs->gconf));
+      bst_param_view_set_object (BST_PARAM_VIEW (prefs->bst_param_view), BSE_OBJECT_ID (prefs->gconf));
+      if (g_type_next_base (BSE_OBJECT_TYPE (prefs->gconf), BSE_TYPE_GCONFIG))
 	{
 	  gtk_widget_show (prefs->bst_param_view);
-	  gtk_notebook_set_page (GTK_NOTEBOOK (prefs->notebook), 0);
+	  gtk_notebook_set_current_page (GTK_NOTEBOOK (prefs->notebook), 0);
 	}
       else
 	gtk_widget_hide (prefs->bst_param_view);
@@ -296,8 +240,8 @@ bst_preferences_default_revert (BstPreferences *prefs)
 #include <errno.h>
 #include "../PKG_config.h"
 static void
-ifactory_print_func (gpointer  user_data,
-		     gchar    *str)
+ifactory_print_func (gpointer     user_data,
+		     const gchar *str)
 {
   BseStorage *storage = user_data;
 
@@ -308,6 +252,39 @@ ifactory_print_func (gpointer  user_data,
     bse_storage_needs_break (storage);
 }
 
+static void
+accel_map_print (gpointer        data,
+		 const gchar    *accel_path,
+		 guint           accel_key,
+		 guint           accel_mods,
+		 gboolean        changed)
+{
+  GString *gstring = g_string_new (changed ? NULL : "; ");
+  BseStorage *storage = data;
+  gchar *tmp, *name;
+
+  g_string_append (gstring, "(gtk_accel_path \"");
+
+  tmp = g_strescape (accel_path, NULL);
+  g_string_append (gstring, tmp);
+  g_free (tmp);
+
+  g_string_append (gstring, "\" \"");
+
+  name = gtk_accelerator_name (accel_key, accel_mods);
+  tmp = g_strescape (name, NULL);
+  g_free (name);
+  g_string_append (gstring, tmp);
+  g_free (tmp);
+
+  g_string_append (gstring, "\")");
+
+  bse_storage_puts (storage, gstring->str);
+  bse_storage_break (storage);
+
+  g_string_free (gstring, TRUE);
+}
+
 BseErrorType
 bst_rc_dump (const gchar *file_name,
 	     BseGConfig  *gconf)
@@ -353,7 +330,8 @@ bst_rc_dump (const gchar *file_name,
   bse_storage_break (storage);
   bse_storage_puts (storage, "(menu-accelerators");
   bse_storage_push_level (storage);
-  gtk_item_factory_dump_items (NULL, TRUE, ifactory_print_func, storage);
+  bse_storage_break (storage);
+  gtk_accel_map_foreach (storage, accel_map_print);
   bse_storage_pop_level (storage);
   bse_storage_handle_break (storage);
   bse_storage_putc (storage, ')');
@@ -400,11 +378,7 @@ bst_rc_parse (const gchar *file_name,
 	    expected_token = bse_object_restore (BSE_OBJECT (gconf), storage);
 	  else if (bse_string_equals ("menu-accelerators", scanner->value.v_identifier))
 	    {
-	      guint symbol_2_token = scanner->config->symbol_2_token;
-
-	      scanner->config->symbol_2_token = FALSE;
-	      gtk_item_factory_parse_rc_scanner (scanner);
-	      scanner->config->symbol_2_token = symbol_2_token;
+	      gtk_accel_map_load_scanner (scanner);
 
 	      if (g_scanner_get_next_token (scanner) != ')')
 		expected_token = ')';
@@ -419,9 +393,51 @@ bst_rc_parse (const gchar *file_name,
   if (expected_token != G_TOKEN_NONE)
     bse_storage_unexp_token (storage, expected_token);
 
-  bse_storage_destroy (storage);
-
   error = scanner->parse_errors >= scanner->max_parse_errors ? BSE_ERROR_PARSE_ERROR : BSE_ERROR_NONE;
 
+  bse_storage_destroy (storage);
+
   return error;
 }
+
+void
+bst_preferences_create_buttons (BstPreferences *prefs,
+				BstDialog      *dialog)
+{
+  GtkWidget *widget;
+
+  g_return_if_fail (BST_IS_PREFERENCES (prefs));
+  g_return_if_fail (BST_IS_DIALOG (dialog));
+  g_return_if_fail (prefs->apply == NULL);
+
+  /* Apply
+   */
+  prefs->apply = g_object_connect (bst_dialog_default_action (dialog, BST_STOCK_APPLY, NULL, NULL),
+				   "swapped_signal::clicked", bst_preferences_apply, prefs,
+				   "swapped_signal::clicked", bst_preferences_save, prefs,
+				   "swapped_signal::destroy", g_nullify_pointer, &prefs->apply,
+				   NULL);
+  gtk_tooltips_set_tip (BST_TOOLTIPS, prefs->apply,
+			"Apply and save the preference values. Some values may only take effect after "
+			"restart. The preference values are locked against modifcation during "
+			"playback.",
+			NULL);
+
+  /* Revert
+   */
+  widget = bst_dialog_action_swapped (dialog, BST_STOCK_REVERT, bst_preferences_revert, prefs);
+  gtk_tooltips_set_tip (BST_TOOLTIPS, widget,
+			"Revert the preference values to the current internal values.",
+			NULL);
+
+  /* Default Revert
+   */
+  widget = bst_dialog_action_swapped (dialog, BST_STOCK_DEFAULT_REVERT, bst_preferences_default_revert, prefs);
+  gtk_tooltips_set_tip (BST_TOOLTIPS, widget,
+			"Revert to hardcoded default values (factory settings).",
+			NULL);
+
+  /* Close
+   */
+  widget = bst_dialog_action (dialog, BST_STOCK_CLOSE, gtk_toplevel_delete, NULL);
+}
