@@ -1,18 +1,18 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1999, 2000 Tim Janik and Red Hat, Inc.
+ * Copyright (C) 1999-2002 Tim Janik and Red Hat, Inc.
  *
- * This library is free software; you can redistribute it and/or
+ * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
+ * Public License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
@@ -20,11 +20,11 @@
 #define __BST_PREFERENCES_H__
 
 #include	"bstparamview.h"
+#include	"bstdialog.h"
 
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
 
@@ -34,7 +34,7 @@ extern "C" {
 #define	BST_PREFERENCES_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), BST_TYPE_PREFERENCES, BstPreferencesClass))
 #define	BST_IS_PREFERENCES(object)	(GTK_CHECK_TYPE ((object), BST_TYPE_PREFERENCES))
 #define	BST_IS_PREFERENCES_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_PREFERENCES))
-#define BST_PREFERENCES_GET_CLASS(obj)  ((BstPreferencesClass*) (((GtkObject*) (obj))->klass))
+#define BST_PREFERENCES_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), BST_TYPE_PREFERENCES, BstPreferencesClass))
 
 
 /* --- structures & typedefs --- */
@@ -50,7 +50,6 @@ struct _BstPreferences
   GtkWidget	*bst_param_view;
   GtkWidget	*notebook;
   GtkWidget	*apply;
-  GtkWidget	*close;
 };
 struct _BstPreferencesClass
 {
@@ -68,6 +67,8 @@ void		 bst_preferences_apply		(BstPreferences *prefs);
 void		 bst_preferences_save		(BstPreferences *prefs);
 void		 bst_preferences_revert		(BstPreferences *prefs);
 void		 bst_preferences_default_revert	(BstPreferences *prefs);
+void		 bst_preferences_create_buttons (BstPreferences *prefs,
+						 BstDialog      *dialog);
 #define	bst_preferences_update	bst_preferences_revert
 
 
@@ -80,7 +81,6 @@ BseErrorType	 bst_rc_parse			(const gchar	*file_name,
 
 
 #ifdef __cplusplus
-#pragma {
 }
 #endif /* __cplusplus */
 
