@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,9 +15,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
-#include        "bstprocedure.h"
+#include "bstprocedure.h"
 
-#include        "bststatusbar.h"
+#include "bststatusbar.h"
+#include "bstdialog.h"
+#include "bsttexttools.h"
+#include <gobject/gvaluecollector.h>
+#include <string.h>
 
 
 /* --- prototypes --- */
@@ -27,8 +31,10 @@ static void     bst_procedure_shell_destroy    (GtkObject              *object);
 
 
 /* --- static variables --- */
-static gpointer                parent_class = NULL;
-static BstProcedureShellClass *bst_procedure_shell_class = NULL;
+static BstProcedureShell *global_proc_shell = NULL;
+static gpointer           parent_class = NULL;
+static GQuark             quark_input_params = 0;
+static GQuark             quark_output_params = 0;
 
 
 /* --- functions --- */
@@ -60,11 +66,11 @@ bst_procedure_shell_get_type (void)
 static void
 bst_procedure_shell_class_init (BstProcedureShellClass *class)
 {
-  GtkObjectClass *object_class;
+  GtkObjectClass *object_class = GTK_OBJECT_CLASS (class);
   
-  object_class = GTK_OBJECT_CLASS (class);
+  quark_output_params = g_quark_from_static_string ("Output Parameters");
+  quark_input_params = g_quark_from_static_string ("Input Parameters");
   
-  bst_procedure_shell_class = class;
   parent_class = gtk_type_class (GTK_TYPE_VBOX);
   
   object_class->destroy = bst_procedure_shell_destroy;
@@ -82,9 +88,6 @@ bst_procedure_shell_init (BstProcedureShell *procedure_shell)
   procedure_shell->in_modal_selection = FALSE;
   procedure_shell->in_execution = FALSE;
   procedure_shell->hide_dialog_on_exec = FALSE;
-  procedure_shell->tooltips = gtk_tooltips_new ();
-  gtk_object_ref (GTK_OBJECT (procedure_shell->tooltips));
-  gtk_object_sink (GTK_OBJECT (procedure_shell->tooltips));
 }
 
 static void
@@ -108,15 +111,12 @@ bst_procedure_shell_destroy (GtkObject *object)
   g_return_if_fail (object != NULL);
   
   shell = BST_PROCEDURE_SHELL (object);
-
+  
   if (shell->in_execution)
     g_warning (G_STRLOC ": destroying procedure shell during execution");
   
   bst_procedure_shell_set_proc (shell, NULL);
   
-  gtk_object_unref (GTK_OBJECT (shell->tooltips));
-  shell->tooltips = NULL;
-  
   GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
@@ -138,15 +138,15 @@ bst_procedure_shell_set_proc (BstProcedureShell *procedure_shell,
   g_return_if_fail (BST_IS_PROCEDURE_SHELL (procedure_shell));
   if (proc)
     g_return_if_fail (BSE_IS_PROCEDURE_CLASS (proc));
-
+  
   if (proc != procedure_shell->proc)
     {
       bst_procedure_shell_destroy_contents (procedure_shell);
       if (procedure_shell->proc)
-	bse_procedure_unref (procedure_shell->proc);
+	g_type_class_unref (procedure_shell->proc);
       procedure_shell->proc = proc;
       if (procedure_shell->proc)
-	bse_procedure_ref (procedure_shell->proc);
+	g_type_class_ref (BSE_PROCEDURE_TYPE (procedure_shell->proc));
       
       bst_procedure_shell_rebuild (procedure_shell);
     }
@@ -168,10 +168,8 @@ bst_procedure_shell_rebuild (BstProcedureShell *shell)
   if (!shell->proc)
     return;
   
-  g_return_if_fail (shell->tooltips != NULL);
-
   proc = shell->proc;
-
+  
   /* main container
    */
   param_box = GTK_WIDGET (shell);
@@ -201,8 +199,8 @@ bst_procedure_shell_rebuild (BstProcedureShell *shell)
    */
   if (proc->blurb)
     {
-      GtkWidget *hbox, *text = bst_wrap_text_create (proc->blurb, TRUE, NULL);
-      
+      GtkWidget *hbox, *text = bst_scroll_text_create (BST_TEXT_VIEW_CENTER, proc->blurb);
+
       hbox = gtk_widget_new (GTK_TYPE_HBOX,
                              "visible", TRUE,
                              NULL);
@@ -211,8 +209,8 @@ bst_procedure_shell_rebuild (BstProcedureShell *shell)
                       "visible", TRUE,
                       "label", "Description",
                       "label_xalign", 0.0,
-                      "width", 1,
-                      "height", 50,
+                      "width_request", 1,
+                      "height_request", 50,
                       "child", hbox,
                       "parent", param_box,
                       NULL);
@@ -220,18 +218,18 @@ bst_procedure_shell_rebuild (BstProcedureShell *shell)
   
   /* parameter fields
    */
-  pspec_array_list = g_slist_prepend (pspec_array_list, proc->out_param_specs);
-  pspec_array_list = g_slist_prepend (pspec_array_list, proc->in_param_specs);
+  pspec_array_list = g_slist_prepend (pspec_array_list, proc->out_pspecs);
+  pspec_array_list = g_slist_prepend (pspec_array_list, proc->in_pspecs);
   for (slist = pspec_array_list; slist; slist = slist->next)
     {
-      BseParamSpec **pspec_p;
+      GParamSpec **pspec_p;
       
       pspec_p = slist->data;
       if (pspec_p)
         while (*pspec_p)
           {
-            if ((*pspec_p)->any.flags & BSE_PARAM_SERVE_GUI &&
-                (*pspec_p)->any.flags & BSE_PARAM_READABLE)
+            if ((*pspec_p)->flags & BSE_PARAM_SERVE_GUI &&
+                (*pspec_p)->flags & BSE_PARAM_READABLE)
               {
                 BstParam *bparam;
                 
@@ -239,10 +237,10 @@ bst_procedure_shell_rebuild (BstProcedureShell *shell)
                                            BSE_TYPE_PROCEDURE,
                                            *pspec_p,
                                            (slist->next
-                                            ? g_quark_from_static_string ("Input Parameters")
-                                            : g_quark_from_static_string ("Output Parameters")),
+					    ? g_quark_to_string (quark_input_params)
+					    : g_quark_to_string (quark_output_params)),
                                            param_box,
-                                           GTK_TOOLTIPS (shell->tooltips));
+                                           BST_TOOLTIPS);
                 shell->bparams = g_slist_append (shell->bparams, bparam);
                 (is_out_param ? shell->n_out_params : shell->n_in_params) += 1;
                 if (is_out_param && !shell->first_out_bparam)
@@ -253,7 +251,7 @@ bst_procedure_shell_rebuild (BstProcedureShell *shell)
       is_out_param++;
     }
   g_slist_free (pspec_array_list);
-
+  
   /* initialize parameter values
    */
   bst_procedure_shell_reset (shell);
@@ -266,7 +264,6 @@ bst_procedure_shell_execute (BstProcedureShell *shell)
   GSList *slist;
   
   g_return_if_fail (BST_IS_PROCEDURE_SHELL (shell));
-  g_return_if_fail (GTK_OBJECT_DESTROYED (shell) == FALSE);
   g_return_if_fail (shell->proc != NULL);
   g_return_if_fail (shell->in_execution == FALSE);
   
@@ -274,31 +271,25 @@ bst_procedure_shell_execute (BstProcedureShell *shell)
   
   gtk_widget_ref (widget);
   
-  /* process any pending GUI updates
-   */
-  gdk_flush ();
-  do
-    g_main_iteration (FALSE);
-  while (g_main_pending ());
-  
   for (slist = shell->first_out_bparam; slist; slist = slist->next)
     bst_param_reset (slist->data);
   bst_procedure_shell_update (shell);
   
-  if (!GTK_OBJECT_DESTROYED (widget))
+  if (widget)
     {
       BseErrorType error;
 
+      /* enable procedure notification */
+      bst_status_bar_catch_procs ();
+
       shell->in_execution = TRUE;
       error = bse_procedure_execvl (shell->proc,
                                     shell->bparams,
                                     shell->first_out_bparam);
       shell->in_execution = FALSE;
-      
-      /* check for recursion
-       */
-      if (error == BSE_ERROR_PROC_BUSY)
-        gdk_beep ();
+
+      /* disable procedure notification */
+      bst_status_bar_uncatch_procs ();
       
       bst_procedure_shell_update (shell);
     }
@@ -329,81 +320,90 @@ bst_procedure_shell_reset (BstProcedureShell *shell)
   for (slist = shell->bparams; slist; slist = slist->next)
     {
       BstParam *bparam = slist->data;
-
+      
+      (n > 0 ? bst_param_set_default : bst_param_reset) (bparam);
       bst_param_set_editable (bparam, n-- > 0);
-      bst_param_reset (bparam);
     }
   shell->n_preset_params = 0;
 }
 
-guint
+void
+bst_procedure_shell_unpreset (BstProcedureShell *shell)
+{
+  GSList *slist;
+  gint n;
+  
+  g_return_if_fail (BST_IS_PROCEDURE_SHELL (shell));
+  
+  for (n = shell->n_in_params, slist = shell->bparams; slist && n; n--, slist = slist->next)
+    {
+      BstParam *bparam = slist->data;
+      
+      if (!bparam->editable)
+	{
+	  bst_param_set_default (bparam);
+	  bst_param_set_editable (bparam, TRUE);
+	}
+    }
+  shell->n_preset_params = 0;
+}
+
+gboolean
 bst_procedure_shell_preset (BstProcedureShell *shell,
-			    gboolean           lock_presets,
-			    GSList            *preset_params)
+			    const gchar       *name,
+			    const GValue      *value,
+			    gboolean           lock_preset)
 {
   BseProcedureClass *proc;
-  GSList *bparam_slist;
+  GSList *slist;
   guint n;
   
   g_return_val_if_fail (BST_IS_PROCEDURE_SHELL (shell), 0);
   g_return_val_if_fail (shell->proc != NULL, 0);
+  g_return_val_if_fail (name != NULL, 0);
+  g_return_val_if_fail (G_IS_VALUE (value), 0);
   
   proc = shell->proc;
-  bparam_slist = shell->bparams;
-  n = preset_params ? shell->n_in_params : 0;
   
   /* ok, this is the really interesting part!
-   * we walk the preset_params list and try to unificate the
-   * parameters with the procedure's in parameters. if we find
-   * a name match and type conversion is sucessfull, the procedure
-   * gets invoked with a predefined parameter
+   * we try to unificate the preset parameter with the procedure's
+   * in parameters. if we find a name match and type conversion
+   * is sucessfull, the procedure gets invoked with a
+   * predefined parameter.
    */
   
-  shell->n_preset_params = 0;
-  while (n--)
+  for (slist = shell->bparams, n = shell->n_in_params; n; slist = slist->next, n--)
     {
-      BstParam *bparam = bparam_slist->data;
-      BseParam *iparam = &bparam->param;
-      GSList *slist;
+      BstParam *bparam = slist->data;
       
-      bst_param_set_editable (bparam, TRUE);
-      for (slist = preset_params; slist; slist = slist->next)
-        {
-          BseParam *pparam = slist->data;
-	  
-          if (strcmp (iparam->pspec->any.name, pparam->pspec->any.name) == 0)
-            {
-              if (bst_param_set_from_other (bparam, pparam))
-                {
+      if (strcmp (bparam->pspec->name, name) == 0)
+	{
+	  if (bst_param_set_value (bparam, value))
+	    {
+	      if (lock_preset && bparam->editable)
+		{
 		  shell->n_preset_params += 1;
-                  if (lock_presets)
-                    bst_param_set_editable (bparam, FALSE);
-                  break;
-                }
-              else
-                g_warning (G_STRLOC ": can't convert preset parameter `%s' from `%s' to `%s'",
-                           pparam->pspec->any.name,
-                           bse_type_name (pparam->pspec->type),
-                           bse_type_name (iparam->pspec->type));
-            }
-        }
-      
-      bparam_slist = bparam_slist->next;
+		  bst_param_set_editable (bparam, FALSE);
+		}
+	      return TRUE;
+	    }
+	  else
+	    g_warning (G_STRLOC ": cannot convert `%s' value to `%s' for `%s' parameter `%s'",
+		       g_type_name (G_VALUE_TYPE (value)),
+		       g_type_name (G_PARAM_SPEC_VALUE_TYPE (bparam->pspec)),
+		       g_type_name (G_PARAM_SPEC_TYPE (bparam->pspec)),
+		       name);
+	}
     }
-
-  /* and reset output parameters
-   */
-  for (; bparam_slist; bparam_slist = bparam_slist->next)
-    bst_param_reset (bparam_slist->data);
   
-  return shell->n_preset_params;
+  return FALSE;
 }
 
 static void
 shell_modal_selection_done (GtkWidget *widget)
 {
   BstProcedureShell *shell = BST_PROCEDURE_SHELL (widget);
-
+  
   shell->in_modal_selection = FALSE;
 }
 
@@ -411,100 +411,149 @@ static void
 shell_hide_on_demand (GtkWidget *widget)
 {
   BstProcedureShell *shell = BST_PROCEDURE_SHELL (widget);
-
+  
   if (shell->hide_dialog_on_exec)
     gtk_toplevel_hide (widget);
 }
 
-GtkWidget*
-bst_procedure_dialog_from_shell (BstProcedureShell *shell,
-				 GtkWidget        **dialog_p)
-{
-  GtkWidget *adialog;
-
-  g_return_val_if_fail (BST_IS_PROCEDURE_SHELL (shell), NULL);
-  g_return_val_if_fail (GTK_OBJECT_DESTROYED (shell) == FALSE, NULL);
-  g_return_val_if_fail (GTK_WIDGET (shell)->parent == NULL, NULL);
-  
-  gtk_widget_show (GTK_WIDGET (shell));
-  
-  adialog = bst_adialog_new (NULL, dialog_p, GTK_WIDGET (shell),
-			     BST_ADIALOG_POPUP_POS | BST_ADIALOG_MODAL,
-			     "title", "BEAST: Procedure",
-			     "object_signal_after::hide", shell_modal_selection_done, shell,
-			     "data_choice::Execute", shell_hide_on_demand, shell,
-			     "data_choice::Execute", bst_procedure_shell_execute, shell,
-			     "default_choice::Execute", NULL, NULL,
-			     "choice::Close", gtk_toplevel_hide, NULL,
-			     NULL);
-
-  return adialog;
-}
-
 BstProcedureShell*
-bst_procedure_dialog_get_shell (GtkWidget *dialog)
+bst_procedure_shell_global (void)
 {
-  GtkWidget *child;
+  if (!global_proc_shell)
+    {
+      GtkWidget *dialog;
+      
+      global_proc_shell = (BstProcedureShell*) bst_procedure_shell_new (NULL);
+      g_object_ref (global_proc_shell);
+      gtk_object_sink (GTK_OBJECT (global_proc_shell));
+      dialog = bst_dialog_new (NULL, NULL, BST_DIALOG_STATUS_SHELL | BST_DIALOG_HIDE_ON_DELETE | BST_DIALOG_MODAL,
+			       "Procedure", NULL);
 
-  g_return_val_if_fail (BST_IS_ADIALOG (dialog), NULL);
+      /* we're the best window to indicate procedure/script progress */
+      g_object_connect (dialog,
+			"signal::show", bst_status_push_progress_window, NULL,
+			"signal::hide", bst_status_pop_progress_window, NULL,
+			NULL);
+      gtk_container_add (GTK_CONTAINER (BST_DIALOG (dialog)->vbox), GTK_WIDGET (global_proc_shell));
+      gtk_widget_show (GTK_WIDGET (global_proc_shell));
 
-  child = bst_adialog_get_child (dialog);
-
-  return BST_IS_PROCEDURE_SHELL (child) ? BST_PROCEDURE_SHELL (child) : NULL;
+      /* actions */
+      bst_dialog_default_action_swapped (BST_DIALOG (dialog),
+					 BST_STOCK_EXECUTE, bst_procedure_shell_execute, global_proc_shell);
+      bst_dialog_action (BST_DIALOG (dialog), BST_STOCK_CLOSE, gtk_toplevel_delete, NULL);
+    }
+  return global_proc_shell;
 }
 
-void
-bst_procedure_dialog_exec_modal (GtkWidget *dialog,
-				 gboolean   force_display)
+static void
+bst_procedure_exec_internal (GType        procedure_type,
+			     const gchar *preset_param,
+			     gboolean     modal,
+			     gboolean     auto_start,
+			     gboolean	  main_loop_recurse,
+			     va_list      var_args)
 {
+  BseProcedureClass *proc = g_type_class_ref (procedure_type);
   BstProcedureShell *shell;
-  GtkWidget *widget;
-  
-  g_return_if_fail (BST_IS_ADIALOG (dialog));
-  g_return_if_fail (GTK_WIDGET_VISIBLE (dialog) == FALSE);
-  g_return_if_fail (GTK_WINDOW (dialog)->modal == TRUE);
+  GtkWidget *dialog;
 
-  shell = bst_procedure_dialog_get_shell (dialog);
-  g_return_if_fail (BST_IS_PROCEDURE_SHELL (shell));
-  g_return_if_fail (shell->in_modal_selection == FALSE);
+  /* structure setup */
+  shell = bst_procedure_shell_global ();
+  dialog = gtk_widget_get_toplevel (GTK_WIDGET (shell));
+  bst_procedure_shell_set_proc (shell, proc);
 
-  widget = GTK_WIDGET (shell);
+  /* set preset parameters */
+  bst_procedure_shell_unpreset (shell);
+  while (preset_param)
+    {
+      GType vtype = va_arg (var_args, GType);
+      gchar *error = NULL;
+      GValue value = { 0, };
 
-  gtk_widget_ref (widget);
-  
-  if (!force_display && shell->n_preset_params == shell->n_in_params && shell->n_out_params == 0)
+      g_value_init (&value, vtype);
+      G_VALUE_COLLECT (&value, var_args, 0, &error);
+      if (error)
+	{
+	  g_warning ("%s: %s", G_STRLOC, error);
+	  g_free (error);
+	  break;
+	}
+      bst_procedure_shell_preset (shell, preset_param, &value, TRUE);
+      g_value_unset (&value);
+      preset_param = va_arg (var_args, const gchar*);
+    }
+
+  if (modal)
+    bst_dialog_add_flags (BST_DIALOG (dialog), BST_DIALOG_MODAL);
+  else
+    bst_dialog_clear_flags (BST_DIALOG (dialog), BST_DIALOG_MODAL);
+
+  /* execution */
+  bst_status_window_push (dialog);
+  g_object_ref (dialog);
+  if (auto_start && shell->n_preset_params == shell->n_in_params && shell->n_out_params == 0)
     bst_procedure_shell_execute (shell);
   else
     {
-      if (shell->n_out_params)
-	{
-	  GtkWidget *sbar = bst_status_bar_ensure (GTK_WINDOW (dialog));
-
-	  shell->hide_dialog_on_exec = FALSE;
-
-	  gtk_widget_show (sbar);
-	}
-      else
-	{
-	  GtkWidget *sbar = bst_status_bar_from_window (GTK_WINDOW (dialog));
-
-	  shell->hide_dialog_on_exec = TRUE;
-
-	  if (sbar)
-	    gtk_widget_hide (sbar);
-	}
-      
-      shell->in_modal_selection = TRUE;
-      gtk_widget_show (dialog);
-
       /* hand control over to user
        */
-      bst_status_window_push (dialog);
-      do
-        g_main_iteration (TRUE);
-      while (shell->in_modal_selection);
-      bst_status_window_pop ();
+      gtk_widget_show_now (dialog);
+      gdk_window_raise (dialog->window);
+      if (main_loop_recurse)
+	{
+	  do
+	    {
+	      GDK_THREADS_LEAVE ();
+	      g_main_iteration (TRUE);
+	      GDK_THREADS_ENTER ();
+	    }
+	  while (GTK_WIDGET_DRAWABLE (dialog));
+	}
     }
-  
-  gtk_widget_unref (widget);
+  bst_status_window_pop ();
+  g_object_unref (dialog);
+
+  g_type_class_unref (proc);
+}
+
+void
+bst_procedure_exec_modal (GType        procedure_type,
+			  const gchar *preset_param,
+			  ...)
+{
+  va_list var_args;
+
+  g_return_if_fail (BSE_TYPE_IS_PROCEDURE (procedure_type));
+
+  va_start (var_args, preset_param);
+  bst_procedure_exec_internal (procedure_type, preset_param, TRUE, TRUE, TRUE, var_args);
+  va_end (var_args);
+}
+
+void
+bst_procedure_exec (GType        procedure_type,
+		    const gchar *preset_param,
+		    ...)
+{
+  va_list var_args;
+
+  g_return_if_fail (BSE_TYPE_IS_PROCEDURE (procedure_type));
+
+  va_start (var_args, preset_param);
+  bst_procedure_exec_internal (procedure_type, preset_param, FALSE, FALSE, FALSE, var_args);
+  va_end (var_args);
+}
+
+void
+bst_procedure_exec_auto (GType        procedure_type,
+			 const gchar *preset_param,
+			 ...)
+{
+  va_list var_args;
+
+  g_return_if_fail (BSE_TYPE_IS_PROCEDURE (procedure_type));
+
+  va_start (var_args, preset_param);
+  bst_procedure_exec_internal (procedure_type, preset_param, FALSE, TRUE, FALSE, var_args);
+  va_end (var_args);
 }
