@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
 
@@ -33,7 +32,7 @@ extern "C" {
 #define BST_PROCEDURE_SHELL_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), BST_TYPE_PROCEDURE_SHELL, BstProcedureShellClass))
 #define BST_IS_PROCEDURE_SHELL(object)	    (GTK_CHECK_TYPE ((object), BST_TYPE_PROCEDURE_SHELL))
 #define BST_IS_PROCEDURE_SHELL_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_PROCEDURE_SHELL))
-#define BST_PROCEDURE_SHELL_GET_CLASS(obj)  ((BstProcedureShellClass*) (((GtkObject*) (obj))->klass))
+#define BST_PROCEDURE_SHELL_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), BST_TYPE_PROCEDURE_SHELL, BstProcedureShellClass))
 
 
 /* --- structures & typedefs --- */
@@ -54,8 +53,6 @@ struct _BstProcedureShell
   guint		     in_modal_selection : 1;
   guint		     in_execution : 1;
   guint		     hide_dialog_on_exec : 1;
-  
-  GtkTooltips	    *tooltips;
 };
 struct _BstProcedureShellClass
 {
@@ -72,23 +69,27 @@ void		   bst_procedure_shell_execute	   (BstProcedureShell *procedure_shell);
 void		   bst_procedure_shell_set_proc	   (BstProcedureShell *procedure_shell,
 						    BseProcedureClass *proc);
 void		   bst_procedure_shell_reset	   (BstProcedureShell *procedure_shell);
-guint		   bst_procedure_shell_preset	   (BstProcedureShell *procedure_shell,
-						    gboolean	       lock_presets,
-						    GSList	      *preset_params);
-
-/* --- dialogs --- */
-GtkWidget*	   bst_procedure_dialog_from_shell (BstProcedureShell *shell,
-						    GtkWidget	     **dialog_p);
-BstProcedureShell* bst_procedure_dialog_get_shell  (GtkWidget	      *dialog);
-void		   bst_procedure_dialog_exec_modal (GtkWidget	      *dialog,
-						    gboolean	       force_display);
-
+void		   bst_procedure_shell_unpreset	   (BstProcedureShell *procedure_shell);
+gboolean	   bst_procedure_shell_preset	   (BstProcedureShell *procedure_shell,
+						    const gchar       *name,
+						    const GValue      *value,
+						    gboolean	       lock_preset);
 
 
+/* --- convenience --- */
+BstProcedureShell* bst_procedure_shell_global	(void);
+void		   bst_procedure_exec		(GType		 procedure_type,
+						 const gchar	*preset_param,
+						 ...);
+void		   bst_procedure_exec_auto	(GType		 procedure_type,
+						 const gchar	*preset_param,
+						 ...);
+void		   bst_procedure_exec_modal	(GType		 procedure_type,
+						 const gchar	*preset_param,
+						 ...);
 
 
 #ifdef __cplusplus
-#pragma {
 }
 #endif /* __cplusplus */
 
