@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
 
@@ -35,7 +34,7 @@ extern "C" {
 #define	BST_SNET_ROUTER_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), BST_TYPE_SNET_ROUTER, BstSNetRouterClass))
 #define	BST_IS_SNET_ROUTER(object)	(GTK_CHECK_TYPE ((object), BST_TYPE_SNET_ROUTER))
 #define	BST_IS_SNET_ROUTER_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_SNET_ROUTER))
-#define BST_SNET_ROUTER_GET_CLASS(obj)	((BstSNetRouterClass*) (((GtkObject*) (obj))->klass))
+#define BST_SNET_ROUTER_GET_CLASS(obj)	(GTK_CHECK_GET_CLASS ((obj), BST_TYPE_SNET_ROUTER, BstSNetRouterClass))
 
 
 /* --- structures & typedefs --- */
@@ -45,17 +44,20 @@ struct _BstSNetRouter
 {
   GnomeCanvas	    parent_object;
 
-  GtkWidget	   *toolbar;
+  BstToolbar	   *toolbar;
   GtkWidget	   *palette;
   GtkAdjustment    *adjustment;
 
-  BseSNet	   *snet;
+  BswProxy	    snet;
 
   BstRadioTools    *rtools;
 
   gdouble           world_x, world_y;
-  guint             ochannel_id;
-  BstCanvasSource  *ocsource;
+  guint		    reshow_palette : 1;
+  guint		    channel_hints : 1;
+  guint		    drag_is_input : 1;
+  guint             drag_channel;
+  BstCanvasSource  *drag_csource;
   GnomeCanvasItem  *tmp_line;
   GSList           *link_list;
 };
@@ -63,26 +65,26 @@ struct _BstSNetRouterClass
 {
   GnomeCanvasClass parent_class;
 
-  GtkTooltips     *tooltips;
+  GtkItemFactory  *popup_factory;
 };
 
 
 /* --- prototypes --- */
-GtkType		 bst_snet_router_get_type	     (void);
-GtkWidget*	 bst_snet_router_new		     (BseSNet	    *snet);
-void		 bst_snet_router_set_snet 	     (BstSNetRouter *router,
-						      BseSNet       *snet);
-void		 bst_snet_router_update		     (BstSNetRouter *snet_router);
-void		 bst_snet_router_rebuild	     (BstSNetRouter *snet_router);
-void		 bst_snet_router_adjust_region	     (BstSNetRouter *snet_router);
-BstCanvasSource* bst_snet_router_csource_from_source (BstSNetRouter *snet_router,
-						      BseSource     *source);
-void		 bst_snet_router_toggle_palette	     (BstSNetRouter *snet_router);
-
+GtkType		 bst_snet_router_get_type	      (void);
+GtkWidget*	 bst_snet_router_new		      (BswProxy	     snet);
+void		 bst_snet_router_set_snet 	      (BstSNetRouter *router,
+						       BswProxy       snet);
+void		 bst_snet_router_update		      (BstSNetRouter *snet_router);
+void		 bst_snet_router_rebuild	      (BstSNetRouter *snet_router);
+void		 bst_snet_router_adjust_region	      (BstSNetRouter *snet_router);
+BstCanvasSource* bst_snet_router_csource_from_source  (BstSNetRouter *snet_router,
+						       BswProxy       source);
+void		 bst_snet_router_toggle_channel_hints (BstSNetRouter *router);
+void		 bst_snet_router_toggle_palette	      (BstSNetRouter *snet_router);
+BstSNetRouter*	 bst_snet_router_build_page	      (BswProxy	     snet);
 
 
 #ifdef __cplusplus
-#pragma {
 }
 #endif /* __cplusplus */
 
