@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
 
@@ -36,7 +35,7 @@ extern "C" {
 #define	BST_SNET_SHELL_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), BST_TYPE_SNET_SHELL, BstSNetShellClass))
 #define	BST_IS_SNET_SHELL(object)      (GTK_CHECK_TYPE ((object), BST_TYPE_SNET_SHELL))
 #define	BST_IS_SNET_SHELL_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_SNET_SHELL))
-#define BST_SNET_SHELL_GET_CLASS(obj)  ((BstSNetShellClass*) (((GtkObject*) (obj))->klass))
+#define BST_SNET_SHELL_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), BST_TYPE_SNET_SHELL, BstSNetShellClass))
 
 
 /* --- structures & typedefs --- */
@@ -48,8 +47,6 @@ struct _BstSNetShell
 
   BstParamView   *param_view;
   BstSNetRouter  *snet_router;
-
-  GtkTooltips	*tooltips;
 };
 struct _BstSNetShellClass
 {
@@ -61,13 +58,11 @@ struct _BstSNetShellClass
 
 /* --- prototypes --- */
 GtkType		bst_snet_shell_get_type		(void);
-GtkWidget*	bst_snet_shell_new		(BseSNet	*snet);
 
 
 
 
 #ifdef __cplusplus
-#pragma {
 }
 #endif /* __cplusplus */
 
