@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,14 +21,13 @@
 #include	"bstdefs.h"
 #include	"bstsupershell.h"
 #include	"bstparamview.h"
-#include	"bstinstrumentview.h"
-#include	"bstpatternview.h"
-#include	"bstplaylist.h"
+#include	"bsttrackview.h"
+#include	"bstpartview.h"
+#include	"bstsnetrouter.h"
 
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
 
@@ -38,7 +37,7 @@ extern "C" {
 #define	BST_SONG_SHELL_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), BST_TYPE_SONG_SHELL, BstSongShellClass))
 #define	BST_IS_SONG_SHELL(object)      (GTK_CHECK_TYPE ((object), BST_TYPE_SONG_SHELL))
 #define	BST_IS_SONG_SHELL_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_SONG_SHELL))
-#define BST_SONG_SHELL_GET_CLASS(obj)  ((BstSongShellClass*) (((GtkObject*) (obj))->klass))
+#define BST_SONG_SHELL_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), BST_TYPE_SONG_SHELL, BstSongShellClass))
 
 
 /* --- structures & typedefs --- */
@@ -49,11 +48,9 @@ struct _BstSongShell
   BstSuperShell	parent_object;
 
   BstParamView   *param_view;
-  BstItemView    *instrument_view;
-  BstItemView    *pattern_view;
-  GtkWidget      *play_list;
-
-  GtkTooltips	*tooltips;
+  BstItemView    *track_view;
+  BstItemView    *part_view;
+  BstSNetRouter  *snet_router;
 };
 struct _BstSongShellClass
 {
@@ -66,13 +63,11 @@ struct _BstSongShellClass
 
 /* --- prototypes --- */
 GtkType		bst_song_shell_get_type		(void);
-GtkWidget*	bst_song_shell_new		(BseSong	*song);
 
 
 
 
 #ifdef __cplusplus
-#pragma {
 }
 #endif /* __cplusplus */
 
