@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
 
@@ -34,7 +33,7 @@ extern "C" {
 #define	BST_SUPER_SHELL_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), BST_TYPE_SUPER_SHELL, BstSuperShellClass))
 #define	BST_IS_SUPER_SHELL(object)	(GTK_CHECK_TYPE ((object), BST_TYPE_SUPER_SHELL))
 #define	BST_IS_SUPER_SHELL_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_SUPER_SHELL))
-#define BST_SUPER_SHELL_GET_CLASS(obj)	((BstSuperShellClass*) (((GtkObject*) (obj))->klass))
+#define BST_SUPER_SHELL_GET_CLASS(obj)	(GTK_CHECK_GET_CLASS ((obj), BST_TYPE_SUPER_SHELL, BstSuperShellClass))
 
 
 /* --- structures & typedefs --- */
@@ -46,7 +45,7 @@ struct _BstSuperShell
 
   GtkAccelGroup	*accel_group;
 
-  BseSuper	*super;
+  BswProxy	 super;
   guint		 name_set_id;
 };
 struct _BstSuperShellClass
@@ -54,9 +53,9 @@ struct _BstSuperShellClass
   GtkVBoxClass	parent_class;
   
   void		(*setup_super)	 (BstSuperShell  *super_shell,
-				  BseSuper	 *super);
+				  BswProxy	  super);
   void		(*release_super) (BstSuperShell  *super_shell,
-				  BseSuper	 *super);
+				  BswProxy	  super);
   void		(*operate)	 (BstSuperShell  *super_shell,
 				  BstOps          op);
   gboolean	(*can_operate)	 (BstSuperShell  *super_shell,
@@ -73,9 +72,8 @@ void		bst_super_shell_operate		(BstSuperShell	*super_shell,
 gboolean	bst_super_shell_can_operate	(BstSuperShell	*super_shell,
 						 BstOps		 op);
 void		bst_super_shell_set_super	(BstSuperShell	*super_shell,
-						 BseSuper	*super);
-BstSuperShell*	bst_super_shell_from_super	(BseSuper	*super);
-void            bst_super_shell_rebuild         (BstSuperShell  *super_shell);
+						 BswProxy	 super);
+BstSuperShell*	bst_super_shell_from_super	(BswProxy	 super);
 void            bst_super_shell_update          (BstSuperShell  *super_shell);
 void            bst_super_shell_update_parent   (BstSuperShell  *super_shell);
 
@@ -84,7 +82,6 @@ void            bst_super_shell_update_parent   (BstSuperShell  *super_shell);
 
 
 #ifdef __cplusplus
-#pragma {
 }
 #endif /* __cplusplus */
 
