@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-1999, 2000-2002 Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,14 +31,15 @@ struct _CEntry
 {
   CEntry  *next;
   GQuark   category;
-  guint    mindex;
-  BseType  type;
-  BseIcon *icon;
+  guint    mindex, lindex;
+  GType    type;
+  BswIcon *icon;
 };
 
 
 /* --- variables --- */
 static CEntry      *cat_entries = NULL;
+static gboolean     cats_need_sort = FALSE;
 
 
 /* --- functions --- */
@@ -67,7 +68,11 @@ category_strip_toplevels (const gchar *category)
       if (p && p[1])
 	return p - category + 1;
     }
-  else if (l > 8 && strncmp (category, "/Source/", 8) == 0)
+  else if (l > 8 && strncmp (category, "/Modules/", 9) == 0)
+    return 9;
+  else if (l > 8 && strncmp (category, "/Scripts/", 9) == 0)
+    return 9;
+  else if (l > 8 && strncmp (category, "/Effect/", 8) == 0)
     return 8;
   else if (l > 6 && strncmp (category, "/Proc/", 6) == 0)
     return 6;
@@ -75,6 +80,23 @@ category_strip_toplevels (const gchar *category)
   return 0;
 }
 
+static guint
+leaf_index (const gchar *string)
+{
+  gboolean in_quote = FALSE;
+  guint pos = 0;
+  const gchar *p;
+
+  for (p = string; *p; p++)
+    switch (*p)
+      {
+      case '\\':	in_quote = TRUE;			break;
+      case '/':		pos = in_quote ? pos : p - string;	/* fall through */
+      default:		in_quote = FALSE;
+      }
+  return pos;
+}
+
 static inline CEntry*
 centry_new (const gchar *caller,
 	    const gchar *category)
@@ -87,13 +109,13 @@ centry_new (const gchar *caller,
   mindex = category_strip_toplevels (category);
   if (!mindex)
     {
-      g_warning ("%s(): unable to add non-conforming category `%s'", caller, category);
+      g_warning ("%s(): refusing to add non-conforming category `%s'", caller, category);
       return NULL;
     }
   quark = g_quark_try_string (category);
   if (quark && centry_find (quark))
     {
-      g_warning ("%s(): unable to re-add existing category `%s'", caller, category);
+      g_warning ("%s(): unable to add category duplicate `%s'", caller, category);
       return NULL;
     }
 
@@ -109,24 +131,41 @@ centry_new (const gchar *caller,
   else
     centry = g_trash_stack_pop (&free_entries);
 
-  /* FIXME: need sorting alphabetically */
   centry->next = cat_entries;
   cat_entries = centry;
   centry->mindex = mindex - 1;
+  centry->lindex = leaf_index (category);
   centry->category = g_quark_from_string (category);
 
+  cats_need_sort = TRUE;
+
   return centry;
 }
 
+static void
+check_type (GType type)
+{
+  if (BSE_TYPE_IS_PROCEDURE (type))
+    {
+      gchar *x = g_strcanon (g_strdup (g_type_name (type)),
+			     G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS "+",
+			     '-');
+      if (strcmp (x, g_type_name (type)) != 0)
+	g_warning ("type name with invalid characters: %s", g_type_name (type));
+      g_free (x);
+    }
+}
+
 void
 bse_categories_register (const gchar *category,
-			 BseType      type)
+			 GType        type)
 {
   CEntry *centry;
 
   g_return_if_fail (category != NULL);
 
   centry = centry_new (G_GNUC_PRETTY_FUNCTION, category);
+  check_type (type);
   if (centry)
     {
       centry->type = type;
@@ -136,7 +175,7 @@ bse_categories_register (const gchar *category,
 
 void
 bse_categories_register_icon (const gchar      *category,
-			      BseType           type,
+			      GType             type,
 			      const BsePixdata *pixdata)
 {
   CEntry *centry;
@@ -145,22 +184,60 @@ bse_categories_register_icon (const gchar      *category,
   g_return_if_fail (pixdata != NULL);
 
   centry = centry_new (G_GNUC_PRETTY_FUNCTION, category);
+  check_type (type);
   if (centry)
     {
       centry->type = type;
       if (pixdata->type && pixdata->width && pixdata->height && pixdata->encoded_pix_data)
 	{
 	  centry->icon = bse_icon_from_pixdata (pixdata); /* static reference */
-	  bse_icon_ref_static (centry->icon);
+	  bsw_icon_ref_static (centry->icon);
 	}
       else
 	centry->icon = NULL;
     }
 }
 
+static gint
+centries_compare (gconstpointer a,
+		  gconstpointer b)
+{
+  const CEntry *e1 = a;
+  const CEntry *e2 = b;
+  gchar *c1 = g_quark_to_string (e1->category);
+  gchar *c2 = g_quark_to_string (e2->category);
+
+  return strcmp (c2, c1);
+}
+
+static void
+cats_sort (void)
+{
+  GSList *slist, *clist = NULL;
+  CEntry *centry, *last;
+
+  if (!cats_need_sort)
+    return;
+
+  for (centry = cat_entries; centry; centry = centry->next)
+    clist = g_slist_prepend (clist, centry);
+  clist = g_slist_sort (clist, centries_compare);
+  last = NULL;
+  for (slist = clist; slist; slist = slist->next)
+    {
+      centry = slist->data;
+      centry->next = last;
+      last = centry;
+    }
+  cat_entries = centry;
+  g_slist_free (clist);
+
+  cats_need_sort = FALSE;
+}
+
 static inline BseCategory*
 categories_match (const gchar *pattern,
-		  BseType      base_type,
+		  GType        base_type,
 		  guint       *n_matches)
 {
   GPatternSpec *pspec;
@@ -175,7 +252,7 @@ categories_match (const gchar *pattern,
       gchar *category = g_quark_to_string (centry->category);
 
       if (g_pattern_match_string (pspec, category) &&
-	  (!base_type || bse_type_conforms_to (centry->type, base_type)))
+	  (!base_type || g_type_is_a (centry->type, base_type)))
 	{
 	  guint i = n_cats;
 	  
@@ -183,6 +260,7 @@ categories_match (const gchar *pattern,
 	  cats = g_renew (BseCategory, cats, n_cats);
 	  cats[i].category = category;
 	  cats[i].mindex = centry->mindex;
+	  cats[i].lindex = centry->lindex;
 	  cats[i].type = centry->type;
 	  cats[i].icon = centry->icon;
 	}
@@ -204,24 +282,28 @@ bse_categories_match (const gchar *pattern,
     *n_matches = 0;
   g_return_val_if_fail (pattern != NULL, NULL);
 
+  cats_sort ();
+
   return categories_match (pattern, 0, n_matches);
 }
 
 BseCategory* /* free result */
 bse_categories_match_typed (const gchar *pattern,
-			    BseType      base_type,
+			    GType        base_type,
 			    guint       *n_matches)
 {
   if (n_matches)
     *n_matches = 0;
   g_return_val_if_fail (pattern != NULL, NULL);
-  g_return_val_if_fail (base_type > BSE_TYPE_NONE, NULL);
+  g_return_val_if_fail (base_type > G_TYPE_NONE, NULL);
+
+  cats_sort ();
 
   return categories_match (pattern, base_type, n_matches);
 }
 
 BseCategory* /* free result */
-bse_categories_from_type (BseType type,
+bse_categories_from_type (GType   type,
 			  guint  *n_categories)
 {
   CEntry *centry;
@@ -237,6 +319,7 @@ bse_categories_from_type (BseType type,
 	cats = g_renew (BseCategory, cats, n_cats);
 	cats[i].category = g_quark_to_string (centry->category);
 	cats[i].mindex = centry->mindex;
+	cats[i].lindex = centry->lindex;
 	cats[i].type = centry->type;
 	cats[i].icon = centry->icon;
       }
