@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-1999, 2000-2002 Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,11 +24,12 @@
 
 /* --- object type macros --- */
 #define	BSE_TYPE_CONTAINER		(BSE_TYPE_ID (BseContainer))
-#define BSE_CONTAINER(object)		(BSE_CHECK_STRUCT_CAST ((object), BSE_TYPE_CONTAINER, BseContainer))
-#define BSE_CONTAINER_CLASS(class)	(BSE_CHECK_CLASS_CAST ((class), BSE_TYPE_CONTAINER, BseContainerClass))
-#define BSE_IS_CONTAINER(object)	(BSE_CHECK_STRUCT_TYPE ((object), BSE_TYPE_CONTAINER))
-#define BSE_IS_CONTAINER_CLASS(class)	(BSE_CHECK_CLASS_TYPE ((class), BSE_TYPE_CONTAINER))
-#define BSE_CONTAINER_GET_CLASS(object) ((BseContainerClass*) (((BseObject*) (object))->bse_struct.bse_class))
+#define BSE_CONTAINER(object)		(G_TYPE_CHECK_INSTANCE_CAST ((object), BSE_TYPE_CONTAINER, BseContainer))
+#define BSE_CONTAINER_CLASS(class)	(G_TYPE_CHECK_CLASS_CAST ((class), BSE_TYPE_CONTAINER, BseContainerClass))
+#define BSE_IS_CONTAINER(object)	(G_TYPE_CHECK_INSTANCE_TYPE ((object), BSE_TYPE_CONTAINER))
+#define BSE_IS_CONTAINER_CLASS(class)	(G_TYPE_CHECK_CLASS_TYPE ((class), BSE_TYPE_CONTAINER))
+#define BSE_CONTAINER_GET_CLASS(object) (G_TYPE_INSTANCE_GET_CLASS ((object), BSE_TYPE_CONTAINER, BseContainerClass))
+#define BSE_CONTAINER_FLAGS_USHIFT	(BSE_SOURCE_FLAGS_USHIFT + 0)
 
 
 /* --- BseContainer object --- */
@@ -45,7 +46,7 @@ struct _BseContainer
 };
 struct _BseContainerClass
 {
-  BseSourceClass parent_class;
+  BseSourceClass  parent_class;
   
   void		(*add_item)		(BseContainer		*container,
 					 BseItem		*item);
@@ -54,56 +55,53 @@ struct _BseContainerClass
   void		(*forall_items)		(BseContainer		*container,
 					 BseForallItemsFunc	 func,
 					 gpointer		 data);
-  guint		(*item_seqid)		(BseContainer		*container,
-					 BseItem		*item);
-  BseItem*	(*get_item)		(BseContainer		*container,
-					 BseType		 item_type,
-					 guint			 seq_id);
+  BseItem*	(*retrieve_child)	(BseContainer		*container,
+					 GType			 child_type,
+					 const gchar		*uname);
+  GSList*	(*context_children)	(BseContainer		*container);
 };
 
 
 /* --- prototypes --- */
 void		bse_container_add_item		(BseContainer	*container,
 						 BseItem	*item);
-BseItem*        bse_container_new_item          (BseContainer   *container,
-						 BseType         item_type,
+gpointer        bse_container_new_item          (BseContainer   *container,
+						 GType           item_type,
 						 const gchar    *first_param_name,
 						 ...);
 void		bse_container_remove_item	(BseContainer	*container,
 						 BseItem	*item);
-void		bse_container_add_item_unrefed	(BseContainer	*container,
-						 BseItem	*item);
 void		bse_container_forall_items	(BseContainer	*container,
 						 BseForallItemsFunc func,
 						 gpointer	 data);
-GList*		bse_container_list_items	(BseContainer	*container);
+BswIterProxy*	bse_container_list_items	(BseContainer	*container);
 guint		bse_container_get_item_seqid	(BseContainer	*container,
 						 BseItem	*item);
 BseItem*	bse_container_get_item		(BseContainer	*container,
-						 BseType	 item_type,
+						 GType  	 item_type,
 						 guint		 seq_id);
 void		bse_container_store_items	(BseContainer	*container,
-						 BseStorage	*storage);
+						 BseStorage	*storage,
+						 const gchar	*restore_func);
 BseItem*	bse_container_lookup_item	(BseContainer	*container,
-						 const gchar	*name);
-BseItem*	bse_container_item_from_handle	(BseContainer	*container,
-						 const gchar	*handle);
-BseItem*	bse_container_item_from_path	(BseContainer	*container,
-						 const gchar	*path);
-gchar* /*fr*/	bse_container_make_item_path	(BseContainer	*container,
-						 BseItem	*item,
-						 gboolean	 persistent);
+						 const gchar	*uname);
+BseItem*	bse_container_retrieve_child	(BseContainer	*container,
+						 const gchar	*type_uname);
+BseItem*	bse_container_resolve_upath	(BseContainer	*container,
+						 const gchar	*upath);
+gchar* /*fr*/	bse_container_make_upath	(BseContainer	*container,
+						 BseItem	*item);
 
 
-/* --- internal fucntions --- */
+/* --- internal functions --- */
 void            bse_container_cross_ref         (BseContainer    *container,
 						 BseItem         *owner,
 						 BseItem         *ref_item,
-						 BseItemCrossFunc destroy_func,
-						 gpointer         data);
+						 BseItemUncross   uncross_func);
 void            bse_container_cross_unref       (BseContainer    *container,
 						 BseItem         *owner,
-						 BseItem         *ref_item);
+						 BseItem         *ref_item,
+						 gboolean	 notify);
 void		bse_container_uncross_item	(BseContainer    *container,
 						 BseItem         *item);
 void		bse_container_cross_forall	(BseContainer	*container,
