@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 static gchar *indent_inc = NULL;
 static guint spacing = 1;
 static FILE *f_out = NULL;
-static BseType root = BSE_TYPE_OBJECT;
+static GType root = 0;
 static gboolean recursion = TRUE;
 static gboolean feature_blurb = FALSE;
 static gboolean feature_channels = FALSE;
@@ -48,17 +48,17 @@ static gboolean feature_channels = FALSE;
 #define	O_KEY_FILL "_"
 
 static void
-show_nodes (BseType type,
-	    BseType sibling,
+show_nodes (GType        type,
+	    GType        sibling,
 	    const gchar *indent)
 {
-  BseType *children;
+  GType   *children;
   guint i;
 
   if (!type)
     return;
 
-  children = bse_type_children (type, NULL);
+  children = g_type_children (type, NULL);
 
   if (type != root)
     for (i = 0; i < spacing; i++)
@@ -68,9 +68,9 @@ show_nodes (BseType type,
 	   indent,
 	   sibling ? O_BRANCH : (type != root ? O_LLEAF : O_SPACE),
 	   O_ESPACE,
-	   bse_type_name (type));
+	   g_type_name (type));
 
-  for (i = strlen (bse_type_name (type)); i <= strlen (indent_inc); i++)
+  for (i = strlen (g_type_name (type)); i <= strlen (indent_inc); i++)
     fputs (O_KEY_FILL, f_out);
 
   if (feature_blurb && bse_type_blurb (type))
@@ -80,17 +80,17 @@ show_nodes (BseType type,
       fputs ("]", f_out);
     }
 
-  if (feature_channels && bse_type_is_a (type, BSE_TYPE_SOURCE))
+  if (feature_channels && g_type_is_a (type, BSE_TYPE_SOURCE))
     {
-      BseSourceClass *class = bse_type_class_ref (type);
+      BseSourceClass *class = g_type_class_ref (type);
       gchar buffer[1024];
 
       sprintf (buffer,
 	       "\t(ichannels %u) (ochannels %u)",
-	       class->n_ichannels,
-	       class->n_ochannels);
+	       class->channel_defs.n_ichannels,
+	       class->channel_defs.n_ochannels);
       fputs (buffer, f_out);
-      bse_type_class_unref (class);
+      g_type_class_unref (class);
     }
 
   fputc ('\n', f_out);
@@ -98,7 +98,7 @@ show_nodes (BseType type,
   if (children && recursion)
     {
       gchar *new_indent;
-      BseType *child;
+      GType   *child;
 
       if (sibling)
 	new_indent = g_strconcat (indent, O_VLINE, indent_inc, NULL);
@@ -124,7 +124,52 @@ show_cats (void)
   for (i = 0; i < n_cats; i++)
     fprintf (f_out, "%s\t(%s)\n",
 	     cats[i].category,
-	     bse_type_name (cats[i].type));
+	     g_type_name (cats[i].type));
+  g_free (cats);
+}
+
+static void
+show_procdoc (void)
+{
+  BseCategory *cats;
+  guint i, n_cats;
+  const gchar *nullstr = ""; // "???";
+
+  cats = bse_categories_match_typed ("*", BSE_TYPE_PROCEDURE, &n_cats);
+  for (i = 0; i < n_cats; i++)
+    {
+      BseProcedureClass *class = g_type_class_ref (cats[i].type);
+      gchar *pname = g_type_name_to_sname (g_type_name (cats[i].type));
+      guint j;
+
+      if (pname[0] == 'b' && pname[1] == 's' && pname[2] == 'e')
+	pname[2] = 'w';
+
+      fprintf (f_out, "/**\n * %s\n", pname);
+      for (j = 0; j < class->n_in_pspecs; j++)
+	{
+	  GParamSpec *pspec = G_PARAM_SPEC (class->in_pspecs[j]);
+
+	  fprintf (f_out, " * @%s: %s\n",
+		   pspec->name,
+		   g_param_spec_get_blurb (pspec) ? g_param_spec_get_blurb (pspec) : nullstr);
+	}
+      for (j = 0; j < class->n_out_pspecs; j++)
+	{
+	  GParamSpec *pspec = G_PARAM_SPEC (class->out_pspecs[j]);
+
+	  fprintf (f_out, " * @Returns: %s: %s\n",
+		   pspec->name,
+		   g_param_spec_get_blurb (pspec) ? g_param_spec_get_blurb (pspec) : nullstr);
+	}
+      if (class->help)
+	fprintf (f_out, " * %s\n", class->help);
+      else if (class->blurb)
+	fprintf (f_out, " * %s\n", class->blurb);
+      fprintf (f_out, " **/\n");
+      g_type_class_unref (class);
+      g_free (pname);
+    }
   g_free (cats);
 }
 
@@ -132,18 +177,20 @@ static gint
 help (gchar *arg)
 {
   fprintf (stderr, "usage: query <qualifier> [-r <type>] [-{i|b} \"\"] [-s #] [-{h|x|y}]\n");
-  fprintf (stderr, "       -r       specifiy root object type\n");
+  fprintf (stderr, "       -r       specifiy root type\n");
+  fprintf (stderr, "       -n       don't descend type tree\n");
+  fprintf (stderr, "       -p       include plugins\n");
+  fprintf (stderr, "       -x       show type blurbs\n");
+  fprintf (stderr, "       -y       show source channels\n");
+  fprintf (stderr, "       -h       guess what ;)\n");
   fprintf (stderr, "       -b       specifiy indent string\n");
   fprintf (stderr, "       -i       specifiy incremental indent string\n");
   fprintf (stderr, "       -s       specifiy line spacing\n");
-  fprintf (stderr, "       -n       no recursion\n");
-  fprintf (stderr, "       -p       include plugins\n");
-  fprintf (stderr, "       -h       guess what ;)\n");
-  fprintf (stderr, "       -x       show type blurbs\n");
-  fprintf (stderr, "       -y       show source channels\n");
   fprintf (stderr, "qualifiers:\n");
-  fprintf (stderr, "       tree     print object tree\n");
+  fprintf (stderr, "       froots   iterate over fundamental roots\n");
+  fprintf (stderr, "       tree     print BSE type tree\n");
   fprintf (stderr, "       cats     print categories\n");
+  fprintf (stderr, "       procdoc  print procedure documentation\n");
 
   return arg != NULL;
 }
@@ -152,14 +199,20 @@ int
 main (gint   argc,
       gchar *argv[])
 {
+  gboolean gen_froots = 0;
   gboolean gen_tree = 0;
   gboolean gen_cats = 0;
+  gboolean gen_procdoc = 0;
   guint i;
   gchar *iindent = "";
   
   f_out = stdout;
 
-  bse_init (&argc, &argv);
+  g_thread_init (NULL);
+
+  bse_init (&argc, &argv, NULL);
+  
+  root = BSE_TYPE_OBJECT;
 
   for (i = 1; i < argc; i++)
     {
@@ -200,7 +253,7 @@ main (gint   argc,
 	{
 	  i++;
 	  if (i < argc)
-	    root = bse_type_from_name (argv[i]);
+	    root = g_type_from_name (argv[i]);
 	}
       else if (strcmp ("-n", argv[i]) == 0)
 	{
@@ -214,6 +267,10 @@ main (gint   argc,
 	{
 	  feature_channels = TRUE;
 	}
+      else if (strcmp ("froots", argv[i]) == 0)
+	{
+	  gen_froots = 1;
+	}
       else if (strcmp ("tree", argv[i]) == 0)
 	{
 	  gen_tree = 1;
@@ -222,22 +279,13 @@ main (gint   argc,
 	{
 	  gen_cats = 1;
 	}
+      else if (strcmp ("procdoc", argv[i]) == 0)
+	{
+	  gen_procdoc = 1;
+	}
       else if (strcmp ("-p", argv[i]) == 0)
 	{
-	  GList *free_list, *list;
-
-	  /* check load BSE plugins to register types */
-	  free_list = bse_plugin_dir_list_files (BSE_PATH_PLUGINS);
-	  for (list = free_list; list; list = list->next)
-	    {
-	      gchar *error, *string = list->data;
-	      
-	      error = bse_plugin_check_load (string);
-	      if (error)
-		g_error ("failed to load plugin \"%s\": %s", string, error);
-	      g_free (string);
-	    }
-	  g_list_free (free_list);
+	  bsw_register_plugins (NULL, TRUE, NULL, NULL, NULL);
 	}
       else if (strcmp ("-h", argv[i]) == 0)
 	{
@@ -250,7 +298,7 @@ main (gint   argc,
       else
 	return help (argv[i]);
     }
-  if (!gen_tree && !gen_cats)
+  if (!gen_froots && !gen_tree && !gen_cats && !gen_procdoc)
     return help (argv[i-1]);
 
   if (!indent_inc)
@@ -264,8 +312,21 @@ main (gint   argc,
 
   if (gen_tree)
     show_nodes (root, 0, iindent);
+  if (gen_froots)
+    {
+      root = ~0;
+      for (i = 0; i < 256; i++)
+	{
+	  gchar *name = g_type_name (i);
+	  
+	  if (name)
+	    show_nodes (i, 0, iindent);
+	}
+    }
   if (gen_cats)
     show_cats ();
-
+  if (gen_procdoc)
+    show_procdoc ();
+  
   return 0;
 }
