@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,10 +49,10 @@ BseTime		bse_time_from_string	(const gchar   *time_string,
 					 BseErrorType	errors[BSE_MAX_DATE_ERRORS]);
 
 
-/* --- notes --- */
+/* --- notes & frequencies --- */
 void	bse_note_examine	(gint		 note,
 				 gint		*octave_p,
-				 guint		*half_tone_p,
+				 guint		*semitone_p,
 				 gboolean	*ht_up_p,
 				 gchar		*letter_p);
 /* return a newly allocated string which describes `note' literally */
@@ -62,16 +62,26 @@ gchar*	bse_note_to_string	(gint		 note);
  */
 gint	bse_note_from_string	(const gchar	*note_string);
 gint	bse_note_from_freq	(gdouble	 freq);
+gint    bse_note_fine_tune_from_note_freq (gint    note,
+					   gdouble freq);
 gdouble	bse_note_to_freq	(gint		 note);
 gdouble	bse_note_to_tuned_freq	(gint		 note,
 				 gint            fine_tune);
 
+/* find match_freq in inclusive_set (NULL acts as wildcard) and don't
+ * find match_freq in exclusive_set (NULL acts as empty set). the sets
+ * have to contain GValues of type G_TYPE_FLOAT.
+ */
+gboolean        bse_value_arrays_match_freq     (gfloat          match_freq,
+						 GValueArray    *inclusive_set,
+						 GValueArray    *exclusive_set);
+gboolean        bse_darrays_match_freq	        (gfloat          match_freq,
+						 GDArray        *inclusive_set,
+						 GDArray        *exclusive_set);
+
 
 /* --- icons --- */
-BseIcon* bse_icon_from_pixdata (const BsePixdata *pixdata);
-BseIcon* bse_icon_ref_static   (BseIcon          *icon);
-BseIcon* bse_icon_ref          (BseIcon          *icon);
-void     bse_icon_unref        (BseIcon          *icon);
+BswIcon* bse_icon_from_pixdata (const BsePixdata *pixdata);
 
 
 /* --- miscellaeous --- */
@@ -90,6 +100,20 @@ void		bse_nullify			(gpointer	*location);
 gchar*		bse_strdup_stripped		(const gchar	*string);
 
 
+/* --- file utils --- */
+void		bse_str_slist_free		(GSList		*slist);
+GSList*		bse_path_pattern_list_matches	(const gchar	*file_pattern,
+						 const gchar	*cwd,
+						 GFileTest	 file_test);
+GSList*		bse_search_path_list_entries	(const gchar	*search_path);
+GSList*		bse_search_path_list_matches	(const gchar	*search_path,
+						 const gchar	*cwd);
+GSList*		bse_search_path_list_files	(const gchar	*search_path,
+						 const gchar	*file_pattern,
+						 const gchar	*cwd,
+						 GFileTest	 file_test);
+
+
 /* --- bbuffer utils --- */
 void	bse_bbuffer_puts	(gchar       	 bbuffer[BSE_BBUFFER_SIZE],
 				 const gchar	*string);
